/* -*- Mode: C; c-basic-offset: 2; indent-tabs-mode: nil -*-
 *
 * Pigment Render base library
 *
 * Copyright © 2006 Fluendo Embedded S.L. <www.fluendo.com>
 * All rights reserved.
 *
 * This file is available under one of two license agreements.
 *
 * This file is licensed under the GPL version 2. See "LICENSE.GPL" in
 * the root of this distribution including a special exception to use
 * Pigment with Fluendo's plugins.
 *
 * The GPL part of Pigment is also available under a commercial licensing
 * agreement from Fluendo. See "LICENSE.Pigment" in the root directory
 * of this distribution package for details on that license.
 */

#ifndef __HAVE_PGM_RENDER_H__
#define __HAVE_PGM_RENDER_H__

#if HAVE_CONFIG_H
#  include "config.h"
#endif /* HAVE_CONFIG_H */

#include "pgmrenderbasewindow.h"
#include "pgmrenderdrawable.h"
#include "pgmrenderevents.h"
#include "pgmrendererror.h"
#include "pgmrendergradient-iface.h"
#include "pgmrenderlayout-iface.h"
#include "pgmrendermedia-iface.h"
#include "pgmrenderpipe3d-iface.h"
#include "pgmrendershadow-iface.h"
#include "pgmrendershape-iface.h"
#include "pgmrenderstroke-iface.h"
#include "pgmrendertext-iface.h"
#include "pgmrenderenumtypes.h"

/* Windows exports */
#if defined(G_OS_WIN32) && (!defined(__MINGW32__))
#  ifdef PGM_RENDER_COMPILATION
#    define PGM_RENDER_EXPORT __declspec(dllexport)
#  else
#    define PGM_RENDER_EXPORT __declspec(dllimport)
#  endif /* PGM_RENDER_COMPILATION */
#else
#  define PGM_RENDER_EXPORT
#endif /* defined(G_OS_WIN32) && (!defined(__MINGW32__)) */

#define INV_255                   0.00392156862745f
#define PLUGIN_PATH_VARIABLE_NAME "PGM_PLUGIN_PATH"

/**
 * PgmRenderPluginCreateWindowFunc:
 *
 * A plugin should provide a pointer to a function of this type in the
 * plugin_desc struct. This function will be called by the user to
 * create a Window.
 *
 * Returns: A new #PgmRenderBaseWindow 
 */
typedef PgmRenderBaseWindow* (*PgmRenderPluginCreateWindowFunc) ();

/**
 * PgmRenderPluginCreateSurfaceFunc:
 *
 * A plugin should provide a pointer to a function of this type in the
 * plugin_desc struct. This function will be called by the user to
 * create a Surface.
 *
 * Returns: A new #PgmRenderDrawable
 */
typedef PgmRenderDrawable* (*PgmRenderPluginCreateSurfaceFunc) ();

/**
 * PgmRenderPluginCreateFontFunc:
 *
 * A plugin should provide a pointer to a function of this type in the
 * plugin_desc struct. This function will be called by the user to
 * create a Font.
 *
 * Returns: A new #PgmRenderDrawable
 */
typedef PgmRenderDrawable* (*PgmRenderPluginCreateFontFunc) ();

typedef struct _PgmRenderPluginDesc PgmRenderPluginDesc;

struct _PgmRenderPluginDesc {
/*   gint                    major_version; */
/*   gint                    minor_version; */
  PgmRenderPluginCreateWindowFunc  create_window;
  PgmRenderPluginCreateSurfaceFunc create_surface;
  PgmRenderPluginCreateFontFunc    create_font;
  gchar                            *name;
  gchar                            *description;
/*   gchar                   *version; */
  gchar                            *license;
/*   gchar                   *source; */
/*   gchar                   *package; */
  gchar                            *origin;
};

#define PGM_RENDER_PLUGIN_DEFINE(create_window,create_surface,create_font,name,description,license,origin)	\
  PgmRenderPluginDesc plugin_desc = {					\
    create_window,							\
    create_surface,							\
    create_font,							\
    name,								\
    description,							\
    license,								\
    origin								\
  };

void pgm_render_init (const gchar *plugin_name);
void pgm_render_deinit ();

/**
 * pgm_render_create_window:
 *
 * Create a new Window.
 */
extern PGM_RENDER_EXPORT PgmRenderPluginCreateWindowFunc pgm_render_create_window;

/**
 * pgm_render_create_surface:
 *
 * Create a new Surface.
 */
extern PGM_RENDER_EXPORT PgmRenderPluginCreateSurfaceFunc pgm_render_create_surface;

/**
 * pgm_render_create_font:
 *
 * Create a new Font.
 */
extern PGM_RENDER_EXPORT PgmRenderPluginCreateFontFunc pgm_render_create_font;

#endif /* __HAVE_PGM_RENDER_H__ */
