/* -*- Mode: C; c-basic-offset: 2; indent-tabs-mode: nil -*-
 *
 * Pigment Render base library
 *
 * Copyright © 2006, 2007 Fluendo Embedded S.L. <www.fluendo.com>
 * All rights reserved.
 *
 * This file is available under one of two license agreements.
 *
 * This file is licensed under the GPL version 2. See "LICENSE.GPL" in
 * the root of this distribution including a special exception to use
 * Pigment with Fluendo's plugins.
 *
 * The GPL part of Pigment is also available under a commercial licensing
 * agreement from Fluendo. See "LICENSE.Pigment" in the root directory
 * of this distribution package for details on that license.
 */

#ifndef __PGM_RENDER_BASE_WINDOW_H__
#define __PGM_RENDER_BASE_WINDOW_H__

/* pgmrenderbasewindow.h and pgmrenderdrawable.h include each other */
typedef struct _PgmRenderBaseWindow PgmRenderBaseWindow;
typedef struct _PgmRenderBaseWindowClass PgmRenderBaseWindowClass;

#include <glib-object.h>
#include "pgmrenderevents.h"
#include "pgmrendererror.h"
#include "pgmrenderdrawable.h"

G_BEGIN_DECLS

#define PGM_RENDER_TYPE_BASE_WINDOW            (pgm_render_base_window_get_type ())
#define PGM_RENDER_BASE_WINDOW(obj)            (G_TYPE_CHECK_INSTANCE_CAST((obj), PGM_RENDER_TYPE_BASE_WINDOW, PgmRenderBaseWindow))
#define PGM_RENDER_BASE_WINDOW_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST((klass), PGM_RENDER_TYPE_BASE_WINDOW, PgmRenderBaseWindowClass))
#define PGM_RENDER_IS_BASE_WINDOW(obj)         (G_TYPE_CHECK_INSTANCE_TYPE((obj), PGM_RENDER_TYPE_BASE_WINDOW))
#define PGM_RENDER_IS_BASE_WINDOW_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE((klass), PGM_RENDER_TYPE_BASE_WINDOW))
#define PGM_RENDER_BASE_WINDOW_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS((obj), PGM_RENDER_TYPE_BASE_WINDOW, PgmRenderBaseWindowClass))

#define PGM_RENDER_BASE_WINDOW_CAST(obj)	    ((PgmRenderBaseWindow *) (obj))
#define PGM_RENDER_BASE_WINDOW_GET_LOCK(basewindow) (PGM_RENDER_BASE_WINDOW_CAST (basewindow)->lock)

/**
 * PGM_RENDER_BASE_WINDOW_LOCK:
 * @basewindow: A #PgmRenderBaseWindow object.
 *
 * Lock the #basewindow instance.
 * FIXME: Need more details.
 */
#define PGM_RENDER_BASE_WINDOW_LOCK(basewindow) g_static_mutex_lock (PGM_RENDER_BASE_WINDOW_GET_LOCK (basewindow))

/**
 * PGM_RENDER_BASE_WINDOW_TRYLOCK:
 * @basewindow: A #PgmRenderBaseWindow object.
 *
 * Try to lock the #basewindow instance if it is not locked yet.
 * FIXME: Need more details.
 */
#define PGM_RENDER_BASE_WINDOW_TRYLOCK(basewindow) g_static_mutex_trylock (PGM_RENDER_BASE_WINDOW_GET_LOCK (basewindow))

/**
 * PGM_RENDER_BASE_WINDOW_UNLOCK:
 * @basewindow: A #PgmRenderBaseWindow object.
 *
 * Unlock the #basewindow instance.
 * FIXME: Need more details.
 */
#define PGM_RENDER_BASE_WINDOW_UNLOCK(basewindow) g_static_mutex_unlock (PGM_RENDER_BASE_WINDOW_GET_LOCK (basewindow))

/**
 * PgmRenderCursor:
 * @PGM_RENDER_CURSOR_RIGHT_ARROW: FIXME
 * @PGM_RENDER_CURSOR_INHERIT: FIXME
 * @PGM_RENDER_CURSOR_NONE: FIXME
 *
 * FIXME
 */
typedef enum {
  PGM_RENDER_CURSOR_LEFT_ARROW = 0,
  PGM_RENDER_CURSOR_INHERIT = 1,
  PGM_RENDER_CURSOR_NONE = 2,
} PgmRenderCursor;

/**
 * PgmRenderWindowScaleMode:
 * @PGM_RENDER_WINDOW_ADJUST: FIXME
 * @PGM_RENDER_WINDOW_CUSTOM: FIXME
 *
 * FIXME
 */
typedef enum {
  PGM_RENDER_WINDOW_ADJUST = 0,
  PGM_RENDER_WINDOW_CUSTOM = 1,
  PGM_RENDER_WINDOW_SCREEN_ADJUST = 2,
} PgmRenderWindowScaleMode;

struct _PgmRenderBaseWindow {
  GObject                  parent;

  GStaticMutex             *lock;
  GList                    *far_drawable;
  GList                    *z_ordered_drawable;
  GList                    *near_drawable;
  gboolean                 dirty;
  gint                     width;
  gint                     height;
  gint                     canvas_width;
  gint                     canvas_height;
  gfloat                   sx;
  gfloat                   sy;
  PgmRenderWindowScaleMode scale_mode;
  PgmRenderCursor          cursor;
};

struct _PgmRenderBaseWindowClass {
  GObjectClass parent_class;

  /*< public >*/
  PgmRenderError (*update)                (PgmRenderBaseWindow *window);
  PgmRenderError (*set_cursor)            (PgmRenderBaseWindow *window, PgmRenderCursor cursor);
  PgmRenderError (*get_cursor)            (PgmRenderBaseWindow *window, PgmRenderCursor *cursor);
  PgmRenderError (*set_color)             (PgmRenderBaseWindow *window, guchar r, guchar g, guchar b, guchar a);
  PgmRenderError (*get_color)             (PgmRenderBaseWindow *window, guchar *r, guchar *g, guchar *b, guchar *a);
  PgmRenderError (*set_title)             (PgmRenderBaseWindow *window, const gchar *window_title, const gchar *icon_title);
  PgmRenderError (*set_size)              (PgmRenderBaseWindow *window, gint width, gint height);
  PgmRenderError (*get_size)              (PgmRenderBaseWindow *window, gint *width, gint *height);
  PgmRenderError (*set_canvas_size)       (PgmRenderBaseWindow *window, gint canvas_width, gint canvas_height);
  PgmRenderError (*get_canvas_size)       (PgmRenderBaseWindow *window, gint *canvas_width, gint *canvas_height);
  PgmRenderError (*set_fullscreen)        (PgmRenderBaseWindow *window, gboolean fullscreen, gint width, gint height);
  PgmRenderError (*get_fullscreen)        (PgmRenderBaseWindow *window, gboolean *fullscreen);
  PgmRenderError (*set_scale)             (PgmRenderBaseWindow *window, gfloat sx, gfloat sy);
  PgmRenderError (*get_scale)             (PgmRenderBaseWindow *window, gfloat *sx, gfloat *sy);
  PgmRenderError (*get_screen_resolution) (PgmRenderBaseWindow *window, gint *width, gint *height);
  PgmRenderError (*get_screen_size)       (PgmRenderBaseWindow *window, gint *width, gint *height);
  PgmRenderError (*get_screen_size_mm)    (PgmRenderBaseWindow *window, gint *width, gint *height);
  PgmRenderError (*set_vertical_sync)     (PgmRenderBaseWindow *window, gboolean vsync);
  PgmRenderError (*get_vertical_sync)     (PgmRenderBaseWindow *window, gboolean *vsync);
  PgmRenderError (*get_events)            (PgmRenderBaseWindow *window, GList **event);
  PgmRenderError (*start_loop)            (PgmRenderBaseWindow *window);
  PgmRenderError (*quit_loop)             (PgmRenderBaseWindow *window);
  PgmRenderError (*add_drawable)          (PgmRenderBaseWindow *window, PgmRenderDrawable *drawable, PgmRenderDrawableOrder order);
  PgmRenderError (*del_drawable)          (PgmRenderBaseWindow *window, PgmRenderDrawable *drawable);
  PgmRenderError (*reorder_drawable)      (PgmRenderBaseWindow *window, PgmRenderDrawable *drawable, gint position);
  PgmRenderError (*regenerate_drawables)  (PgmRenderBaseWindow *window);
  PgmRenderError (*set_scale_mode)        (PgmRenderBaseWindow *window, PgmRenderWindowScaleMode scale_mode);
  PgmRenderError (*get_scale_mode)        (PgmRenderBaseWindow *window, PgmRenderWindowScaleMode *scale_mode);
};

GType                pgm_render_base_window_get_type              (void) G_GNUC_CONST;
PgmRenderBaseWindow *pgm_render_base_window_new                   (void);

PgmRenderError       pgm_render_base_window_update                (PgmRenderBaseWindow *window);
PgmRenderError       pgm_render_base_window_set_cursor            (PgmRenderBaseWindow *window, PgmRenderCursor cursor);
PgmRenderError       pgm_render_base_window_get_cursor            (PgmRenderBaseWindow *window, PgmRenderCursor *cursor);
PgmRenderError       pgm_render_base_window_set_color             (PgmRenderBaseWindow *window, guchar r, guchar g, guchar b, guchar a);
PgmRenderError       pgm_render_base_window_get_color             (PgmRenderBaseWindow *window, guchar *r, guchar *g, guchar *b, guchar *a);
PgmRenderError       pgm_render_base_window_set_title             (PgmRenderBaseWindow *window, const gchar *window_title, const gchar *icon_title);
PgmRenderError       pgm_render_base_window_set_size              (PgmRenderBaseWindow *window, gint width, gint height);
PgmRenderError       pgm_render_base_window_get_size              (PgmRenderBaseWindow *window, gint *width, gint *height);
PgmRenderError       pgm_render_base_window_set_canvas_size       (PgmRenderBaseWindow *window, gint canvas_width, gint canvas_height);
PgmRenderError       pgm_render_base_window_get_canvas_size       (PgmRenderBaseWindow *window, gint *canvas_width, gint *canvas_height);
PgmRenderError       pgm_render_base_window_set_fullscreen        (PgmRenderBaseWindow *window, gboolean fullscreen, gint width, gint height);
PgmRenderError       pgm_render_base_window_get_fullscreen        (PgmRenderBaseWindow *window, gboolean *fullscreen);
PgmRenderError       pgm_render_base_window_set_scale             (PgmRenderBaseWindow *window, gfloat sx, gfloat sy);
PgmRenderError       pgm_render_base_window_get_scale             (PgmRenderBaseWindow *window, gfloat *sx, gfloat *sy);
PgmRenderError       pgm_render_base_window_get_screen_resolution (PgmRenderBaseWindow *window, gint *width, gint *height);
PgmRenderError       pgm_render_base_window_get_screen_size       (PgmRenderBaseWindow *window, gint *width, gint *height);
PgmRenderError       pgm_render_base_window_get_screen_size_mm    (PgmRenderBaseWindow *window, gint *width, gint *height);
PgmRenderError       pgm_render_base_window_set_vertical_sync     (PgmRenderBaseWindow *window, gboolean vsync);
PgmRenderError       pgm_render_base_window_get_vertical_sync     (PgmRenderBaseWindow *window, gboolean *vsync);
PgmRenderError       pgm_render_base_window_get_events            (PgmRenderBaseWindow *window, GList **events);
PgmRenderError       pgm_render_base_window_start_loop            (PgmRenderBaseWindow *window);
PgmRenderError       pgm_render_base_window_quit_loop             (PgmRenderBaseWindow *window);
PgmRenderError       pgm_render_base_window_add_drawable          (PgmRenderBaseWindow *window, PgmRenderDrawable *drawable, PgmRenderDrawableOrder order);
PgmRenderError       pgm_render_base_window_del_drawable          (PgmRenderBaseWindow *window, PgmRenderDrawable *drawable);
PgmRenderError       pgm_render_base_window_reorder_drawable      (PgmRenderBaseWindow *window, PgmRenderDrawable *drawable, gint position);
PgmRenderError       pgm_render_base_window_regenerate_drawables  (PgmRenderBaseWindow *window);
PgmRenderError       pgm_render_base_window_set_scale_mode        (PgmRenderBaseWindow *window, PgmRenderWindowScaleMode scale_mode);
PgmRenderError       pgm_render_base_window_get_scale_mode        (PgmRenderBaseWindow *window, PgmRenderWindowScaleMode *scale_mode);

G_END_DECLS

#endif /* __PGM_RENDER_BASE_WINDOW_H__ */
