/* -*- Mode: C; c-basic-offset: 2; indent-tabs-mode: nil -*-
 *
 * Pigment Render base library
 *
 * Copyright © 2006 Fluendo Embedded S.L. <www.fluendo.com>
 * All rights reserved.
 *
 * This file is available under one of two license agreements.
 *
 * This file is licensed under the GPL version 2. See "LICENSE.GPL" in
 * the root of this distribution including a special exception to use
 * Pigment with Fluendo's plugins.
 *
 * The GPL part of Pigment is also available under a commercial licensing
 * agreement from Fluendo. See "LICENSE.Pigment" in the root directory
 * of this distribution package for details on that license.
 */

/**
 * SECTION:pgmrenderevents
 * @short_description: events functionality.
 *
 * 
 */

#ifdef HAVE_CONFIG_H
#  include "config.h"
#endif

#include "pgmrenderevents.h"

/**
 * pgm_render_event_new:
 * @type: The type of event.
 *
 * Creates a new #PgmRenderEvent of the specified type.
 *
 * Return value: A newly allocated #PgmRenderEvent.
 */
PgmRenderEvent *
pgm_render_event_new (PgmRenderEventType type)
{
  PgmRenderEvent *new_event;

  new_event = g_new0 (PgmRenderEvent, 1);
  new_event->any.type = type;
  /* default source is 0: events generated by Pigment
   * should be overriden depending on the type of source */
  new_event->any.source = 0;

  return new_event;
}

/**
 * pgm_render_event_copy:
 * @event: A #PgmRenderEvent.
 *
 * Copies @event.
 *
 * Return value: A newly allocated #PgmRenderEvent
 */
PgmRenderEvent *
pgm_render_event_copy (PgmRenderEvent *event)
{
  PgmRenderEvent *new_event;

  g_return_val_if_fail (event != NULL, NULL);

  new_event = g_new (PgmRenderEvent, 1);
  *new_event = *event;

  return new_event;
}

/**
 * pgm_render_event_free:
 * @event: A #PgmRenderEvent.
 *
 * Frees all resources used by @event.
 */
void
pgm_render_event_free (PgmRenderEvent *event)
{
  if (!event)
    return;

  g_free (event);
}

GType
pgm_render_event_get_type (void)
{
  static GType our_type = 0;

  if (!our_type)
    our_type = g_boxed_type_register_static
      ("PgmRenderEvent", (GBoxedCopyFunc) pgm_render_event_copy,
       (GBoxedFreeFunc) pgm_render_event_free);

  return our_type;
}
