/* -*- Mode: C; c-basic-offset: 2; indent-tabs-mode: nil -*-
 *
 * Pigment Render base library
 *
 * Copyright © 2006 Fluendo Embedded S.L. <www.fluendo.com>
 * All rights reserved.
 *
 * This file is available under one of two license agreements.
 *
 * This file is licensed under the GPL version 2. See "LICENSE.GPL" in
 * the root of this distribution including a special exception to use
 * Pigment with Fluendo's plugins.
 *
 * The GPL part of Pigment is also available under a commercial licensing
 * agreement from Fluendo. See "LICENSE.Pigment" in the root directory
 * of this distribution package for details on that license.
 */

#ifndef __PGM_RENDER_MEDIA_IFACE_H__
#define __PGM_RENDER_MEDIA_IFACE_H__

#ifdef G_OS_WIN32
typedef struct _GstBuffer GstBuffer;
#endif /* G_OS_WIN32 */

#include "pgmrendererror.h"
#include "pgmrenderdrawable.h"
#include <glib-object.h>

G_BEGIN_DECLS

#define PGM_RENDER_TYPE_MEDIA_IFACE             (pgm_render_media_iface_get_type ())
#define PGM_RENDER_MEDIA_IFACE(obj)             (G_TYPE_CHECK_INSTANCE_CAST((obj), PGM_RENDER_TYPE_MEDIA_IFACE, PgmRenderMediaIFace))
#define PGM_RENDER_MEDIA_IFACE_CLASS(vtable)    (G_TYPE_CHECK_CLASS_CAST((vtable), PGM_RENDER_TYPE_MEDIA_IFACE, GTypeInterface))
#define PGM_RENDER_IS_MEDIA_IFACE(obj)          (G_TYPE_CHECK_INSTANCE_TYPE((obj), PGM_RENDER_TYPE_MEDIA_IFACE))
#define PGM_RENDER_IS_MEDIA_IFACE_CLASS(vtable) (G_TYPE_CHECK_CLASS_TYPE((vtable), PGM_RENDER_TYPE_MEDIA_IFACE))
#define PGM_RENDER_MEDIA_IFACE_GET_CLASS(inst)  (G_TYPE_INSTANCE_GET_INTERFACE((inst), PGM_RENDER_TYPE_MEDIA_IFACE, PgmRenderMediaIFaceClass))

/**
 * PgmRenderMediaInterpType:
 * @PGM_RENDER_MEDIA_NEAREST: The nearest neighbour sampling. This is the
 *                            fastest and lowest quality mode.
 * @PGM_RENDER_MEDIA_BILINEAR: The bilinear sampling. This is the slowest
 *                             and highest quality mode.
 *
 * The different image interpolation types.
 */
typedef enum {
  PGM_RENDER_MEDIA_NEAREST,
  PGM_RENDER_MEDIA_BILINEAR
} PgmRenderMediaInterpType;

/**
 * PgmRenderMediaPixelFormat:
 * @PGM_RENDER_MEDIA_RGB: FIXME
 * @PGM_RENDER_MEDIA_BGR: FIXME
 * @PGM_RENDER_MEDIA_RGBA: FIXME
 * @PGM_RENDER_MEDIA_BGRA: FIXME
 * @PGM_RENDER_MEDIA_I420: FIXME
 * @PGM_RENDER_MEDIA_YV12: FIXME
 * @PGM_RENDER_MEDIA_UYVY: FIXME
 * @PGM_RENDER_MEDIA_YUYV: FIXME
 *
 * The different image pixel formats.
 */
typedef enum {
  PGM_RENDER_MEDIA_RGB  = 0,
  PGM_RENDER_MEDIA_BGR  = 1,
  PGM_RENDER_MEDIA_RGBA = 2,
  PGM_RENDER_MEDIA_BGRA = 3,
  PGM_RENDER_MEDIA_I420 = 4,
  PGM_RENDER_MEDIA_YV12 = 5,
  PGM_RENDER_MEDIA_UYVY = 6,
  PGM_RENDER_MEDIA_YUYV = 7
} PgmRenderMediaPixelFormat;

typedef struct _PgmRenderMediaIFace      PgmRenderMediaIFace;
typedef struct _PgmRenderMediaIFaceClass PgmRenderMediaIFaceClass;

struct _PgmRenderMediaIFaceClass {
  GTypeInterface parent;

  PgmRenderError (*set_color)               (PgmRenderMediaIFace *media, guchar r, guchar g, guchar b, guchar a);
  PgmRenderError (*get_color)               (PgmRenderMediaIFace *media, guchar *r, guchar *g, guchar *b, guchar *a);
  PgmRenderError (*set_image)               (PgmRenderMediaIFace *media, PgmRenderMediaPixelFormat format, gint width, gint height, gint stride, gconstpointer data);
  PgmRenderError (*set_gst_image)           (PgmRenderMediaIFace *media, PgmRenderMediaPixelFormat format, gint width, gint height, gint stride, GstBuffer *buffer);
  PgmRenderError (*set_image_from_drawable) (PgmRenderMediaIFace *media, PgmRenderDrawable *drawable);
  PgmRenderError (*del_image)               (PgmRenderMediaIFace *media);
  PgmRenderError (*set_interp_type)         (PgmRenderMediaIFace *media, PgmRenderMediaInterpType type);
  PgmRenderError (*get_interp_type)         (PgmRenderMediaIFace *media, PgmRenderMediaInterpType *type);
};

GType          pgm_render_media_iface_get_type          (void) G_GNUC_CONST;
PgmRenderError pgm_render_media_set_color               (PgmRenderMediaIFace *media, guchar r, guchar g, guchar b, guchar a);
PgmRenderError pgm_render_media_get_color               (PgmRenderMediaIFace *media, guchar *r, guchar *g, guchar *b, guchar *a);
PgmRenderError pgm_render_media_set_image               (PgmRenderMediaIFace *media, PgmRenderMediaPixelFormat format, gint width, gint height, gint stride, gconstpointer data);
PgmRenderError pgm_render_media_set_gst_image           (PgmRenderMediaIFace *media, PgmRenderMediaPixelFormat format, gint width, gint height, gint stride, GstBuffer *buffer);
PgmRenderError pgm_render_media_set_image_from_drawable (PgmRenderMediaIFace *media, PgmRenderDrawable *drawable);
PgmRenderError pgm_render_media_del_image               (PgmRenderMediaIFace *media);
PgmRenderError pgm_render_media_set_interp_type         (PgmRenderMediaIFace *media, PgmRenderMediaInterpType type);
PgmRenderError pgm_render_media_get_interp_type         (PgmRenderMediaIFace *media, PgmRenderMediaInterpType *type);

G_END_DECLS

#endif /* __PGM_RENDER_MEDIA_IFACE_H__ */
