/* -*- Mode: C; c-basic-offset: 2; indent-tabs-mode: nil -*-
 *
 * Pigment Render base library
 *
 * Copyright © 2006 Fluendo Embedded S.L. <www.fluendo.com>
 * All rights reserved.
 *
 * This file is available under one of two license agreements.
 *
 * This file is licensed under the GPL version 2. See "LICENSE.GPL" in
 * the root of this distribution including a special exception to use
 * Pigment with Fluendo's plugins.
 *
 * The GPL part of Pigment is also available under a commercial licensing
 * agreement from Fluendo. See "LICENSE.Pigment" in the root directory
 * of this distribution package for details on that license.
 */

/**
 * SECTION:pgmrendershape-iface
 * @short_description: An interface for drawing different shapes.
 *
 * #PgmRenderShapeIFace is an interface to add different shape drawing
 * capabilities to an object.
 */

#ifdef HAVE_CONFIG_H
#  include "config.h"
#endif

#include "pgmrendershape-iface.h"

static void pgm_render_shape_iface_base_init (gpointer g_class);

static void
pgm_render_shape_iface_base_init (gpointer g_class)
{
  static gboolean initialized = FALSE;

  if (! initialized)
    {
      initialized = TRUE;
    }
}

GType
pgm_render_shape_iface_get_type (void)
{
  static GType type = 0;

  if (! type)
    {
      static const GTypeInfo info = {
	sizeof (PgmRenderShapeIFaceClass),
	pgm_render_shape_iface_base_init, /* base init */
	NULL, /* base finalize */
	NULL, /* class_init */
	NULL, /* class finalize */
	NULL, /* class data */
	0,
	0,    /* n_preallocs */
	NULL, /* instance init */
      };
      type = g_type_register_static (G_TYPE_INTERFACE, "PgmRenderShapeIFace",
				     &info, 0);
      g_type_interface_add_prerequisite (type, G_TYPE_OBJECT);
    }

  return type;
}

/**
 * pgm_render_shape_set_arc:
 * @shape: A #PgmRenderShapeIFace object.
 * @width: The width of the arc. 0 by default.
 * @height: The height of the arc. 0 by default
 *
 * Sets the arc of #stroke to (#width, #height). The arc is assigned
 * to each angle of the surface.
 *
 * Returns: An error status.
 */
PgmRenderError
pgm_render_shape_set_arc (PgmRenderShapeIFace *shape, gfloat width,
			  gfloat height)
{
  g_return_val_if_fail (PGM_RENDER_IS_SHAPE_IFACE (shape), PGM_RENDER_ERROR_X);

  return PGM_RENDER_SHAPE_IFACE_GET_CLASS
    (shape)->set_arc (shape, width, height);
}

/**
 * pgm_render_shape_get_arc:
 * @shape: A #PgmRenderShapeIFace object.
 * @width: The width of the arc to fill.
 * @height: The height of the arc to fill.
 *
 * Retrieves the (#width, #height) arc of #stroke.
 *
 * Returns: An error status.
 */
PgmRenderError
pgm_render_shape_get_arc (PgmRenderShapeIFace *shape, gfloat *width,
			  gfloat *height)
{
  g_return_val_if_fail (PGM_RENDER_IS_SHAPE_IFACE (shape), PGM_RENDER_ERROR_X);

  return PGM_RENDER_SHAPE_IFACE_GET_CLASS
    (shape)->get_arc (shape, width, height);
}
