/* -*- Mode: C; c-basic-offset: 2; indent-tabs-mode: nil -*-
 *
 * Pigment Render base library
 *
 * Copyright © 2006 Fluendo Embedded S.L. <www.fluendo.com>
 * All rights reserved.
 *
 * This file is available under one of two license agreements.
 *
 * This file is licensed under the GPL version 2. See "LICENSE.GPL" in
 * the root of this distribution including a special exception to use
 * Pigment with Fluendo's plugins.
 *
 * The GPL part of Pigment is also available under a commercial licensing
 * agreement from Fluendo. See "LICENSE.Pigment" in the root directory
 * of this distribution package for details on that license.
 */

#ifndef __PGM_RENDER_STROKE_IFACE_H__
#define __PGM_RENDER_STROKE_IFACE_H__

#include "pgmrendererror.h"
#include <glib-object.h>

G_BEGIN_DECLS

#define PGM_RENDER_TYPE_STROKE_IFACE             (pgm_render_stroke_iface_get_type ())
#define PGM_RENDER_STROKE_IFACE(obj)             (G_TYPE_CHECK_INSTANCE_CAST((obj), PGM_RENDER_TYPE_STROKE_IFACE, PgmRenderStrokeIFace))
#define PGM_RENDER_STROKE_IFACE_CLASS(vtable)    (G_TYPE_CHECK_CLASS_CAST((vtable), PGM_RENDER_TYPE_STROKE_IFACE, GTypeInterface))
#define PGM_RENDER_IS_STROKE_IFACE(obj)          (G_TYPE_CHECK_INSTANCE_TYPE((obj), PGM_RENDER_TYPE_STROKE_IFACE))
#define PGM_RENDER_IS_STROKE_IFACE_CLASS(vtable) (G_TYPE_CHECK_CLASS_TYPE((vtable), PGM_RENDER_TYPE_STROKE_IFACE))
#define PGM_RENDER_STROKE_IFACE_GET_CLASS(inst)  (G_TYPE_INSTANCE_GET_INTERFACE((inst), PGM_RENDER_TYPE_STROKE_IFACE, PgmRenderStrokeIFaceClass))

/**
 * PgmRenderStrokePattern:
 * @PGM_RENDER_STROKE_CONTINUOUS: A linear stroke pattern.
 * @PGM_RENDER_STROKE_DASHED: A dashed stroke pattern.
 *
 * The different stroke pattern.
 */
typedef enum {
  PGM_RENDER_STROKE_CONTINUOUS,
  PGM_RENDER_STROKE_DASHED
} PgmRenderStrokePattern;

typedef struct _PgmRenderStrokeIFace      PgmRenderStrokeIFace;
typedef struct _PgmRenderStrokeIFaceClass PgmRenderStrokeIFaceClass;

struct _PgmRenderStrokeIFaceClass {
  GTypeInterface parent;

  PgmRenderError (*set_width)   (PgmRenderStrokeIFace *stroke, gfloat width);
  PgmRenderError (*get_width)   (PgmRenderStrokeIFace *stroke, gfloat *width);
  PgmRenderError (*set_pattern) (PgmRenderStrokeIFace *stroke, PgmRenderStrokePattern pattern);
  PgmRenderError (*get_pattern) (PgmRenderStrokeIFace *stroke, PgmRenderStrokePattern *pattern);
  PgmRenderError (*set_factor)  (PgmRenderStrokeIFace *stroke, gint factor);
  PgmRenderError (*get_factor)  (PgmRenderStrokeIFace *stroke, gint *factor);
  PgmRenderError (*set_color)   (PgmRenderStrokeIFace *stroke, guchar r, guchar g, guchar b, guchar a);
  PgmRenderError (*get_color)   (PgmRenderStrokeIFace *stroke, guchar *r, guchar *g, guchar *b, guchar *a);
};

GType          pgm_render_stroke_iface_get_type (void) G_GNUC_CONST;
PgmRenderError pgm_render_stroke_set_width      (PgmRenderStrokeIFace *stroke, gfloat width);
PgmRenderError pgm_render_stroke_get_width      (PgmRenderStrokeIFace *stroke, gfloat *width);
PgmRenderError pgm_render_stroke_set_pattern    (PgmRenderStrokeIFace *stroke, PgmRenderStrokePattern pattern);
PgmRenderError pgm_render_stroke_get_pattern    (PgmRenderStrokeIFace *stroke, PgmRenderStrokePattern *pattern);
PgmRenderError pgm_render_stroke_set_factor     (PgmRenderStrokeIFace *stroke, gint factor);
PgmRenderError pgm_render_stroke_get_factor     (PgmRenderStrokeIFace *stroke, gint *factor);
PgmRenderError pgm_render_stroke_set_color      (PgmRenderStrokeIFace *stroke, guchar r, guchar g, guchar b, guchar a);
PgmRenderError pgm_render_stroke_get_color      (PgmRenderStrokeIFace *stroke, guchar *r, guchar *g, guchar *b, guchar *a);

G_END_DECLS

#endif /* __PGM_RENDER_STROKE_IFACE_H__ */
