/* -*- Mode: C; c-basic-offset: 2; indent-tabs-mode: nil -*-
 *
 * Pigment Render OpenGL 1.2 plugin
 *
 * Copyright © 2006, 2007 Fluendo Embedded S.L. <www.fluendo.com>
 * All rights reserved.
 *
 * This file is available under one of two license agreements.
 *
 * This file is licensed under the GPL version 2. See "LICENSE.GPL" in
 * the root of this distribution including a special exception to use
 * Pigment with Fluendo's plugins.
 *
 * The GPL part of Pigment is also available under a commercial licensing
 * agreement from Fluendo. See "LICENSE.Pigment" in the root directory
 * of this distribution package for details on that license.
 *
 * The OpenGL extensions loading code have been inspired by the Glitz
 * image compositing library <www.freedesktop.org/wiki/Software/glitz>.
 * Copyright © 2004, David Reveman, Peter Nilsson
 *
 * Author: Loïc Molinari <loic@fluendo.com>
 */

#ifndef __PGM_RENDER_GL_H__
#define __PGM_RENDER_GL_H__

typedef struct _PgmRenderWglContext PgmRenderWglContext;
typedef struct _PgmRenderGlxContext PgmRenderGlxContext;
typedef struct _PgmRenderGl1Window PgmRenderGl1Window;

#ifdef HAVE_CONFIG_H
#  include "config.h"
#endif /* HAVE_CONFIG_H */
#include <pgmrender.h>
#include <stddef.h>
#include <gst/gst.h>
#include <gst/video/video.h>
#if defined(G_OS_WIN32)
#  include "pgmrender-wgl.h"
#  define PGM_RENDER_GL_API_ATTRIBUTE __stdcall
#elif defined(G_OS_UNIX)
#  include "pgmrender-glx.h"
#  define PGM_RENDER_GL_API_ATTRIBUTE
#endif /* defined(G_OS_UNIX) */

/* The OpenGL feature masks available at runtime */
#define PGM_RENDER_GL_FEATURE_TEXTURE_RECTANGLE        (1L <<  0)
#define PGM_RENDER_GL_FEATURE_TEXTURE_NON_POWER_OF_TWO (1L <<  1)
#define PGM_RENDER_GL_FEATURE_TEXTURE_MIRRORED_REPEAT  (1L <<  2)
#define PGM_RENDER_GL_FEATURE_TEXTURE_BORDER_CLAMP     (1L <<  3)
#define PGM_RENDER_GL_FEATURE_MULTISAMPLE              (1L <<  4)
#define PGM_RENDER_GL_FEATURE_MULTISAMPLE_FILTER_HINT  (1L <<  5)
#define PGM_RENDER_GL_FEATURE_MULTITEXTURE             (1L <<  6)
#define PGM_RENDER_GL_FEATURE_TEXTURE_ENV_COMBINE      (1L <<  7)
#define PGM_RENDER_GL_FEATURE_TEXTURE_ENV_DOT3         (1L <<  8)
#define PGM_RENDER_GL_FEATURE_FRAGMENT_PROGRAM         (1L <<  9)
#define PGM_RENDER_GL_FEATURE_VERTEX_BUFFER_OBJECT     (1L << 10)
#define PGM_RENDER_GL_FEATURE_PIXEL_BUFFER_OBJECT      (1L << 11)
#define PGM_RENDER_GL_FEATURE_PER_COMPONENT_RENDERING  (1L << 12)
#define PGM_RENDER_GL_FEATURE_BLEND_COLOR              (1L << 13)
#define PGM_RENDER_GL_FEATURE_PACKED_PIXELS            (1L << 14)
#define PGM_RENDER_GL_FEATURE_MULTI_DRAW_ARRAYS        (1L << 15)
#define PGM_RENDER_GL_FEATURE_FRAMEBUFFER_OBJECT       (1L << 16)
#define PGM_RENDER_GL_FEATURE_COPY_SUB_BUFFER          (1L << 17)
#define PGM_RENDER_GL_FEATURE_SHADING_LANGUAGE         (1L << 18)
#define PGM_RENDER_GL_FEATURE_SHADER_OBJECTS           (1L << 19)
#define PGM_RENDER_GL_FEATURE_FRAGMENT_SHADER          (1L << 20)
#define PGM_RENDER_GL_FEATURE_VERTEX_SHADER            (1L << 21)
#define PGM_RENDER_GL_FEATURE_GLSL                     (1L << 22)

#define PGM_RENDER_GL_FALSE 0x0
#define PGM_RENDER_GL_TRUE  0x1

#define PGM_RENDER_GL_NO_ERROR          0x0
#define PGM_RENDER_GL_INVALID_OPERATION 0x0502

#define PGM_RENDER_GL_VENDOR                      0x1F00
#define PGM_RENDER_GL_RENDERER                    0x1F01
#define PGM_RENDER_GL_SHADING_LANGUAGE_VERSION    0x8B8C
#define PGM_RENDER_GL_VERSION                     0x1F02
#define PGM_RENDER_GL_EXTENSIONS                  0x1F03

#define PGM_RENDER_GL_UNSIGNED_BYTE               0x1401
#define PGM_RENDER_GL_UNSIGNED_BYTE_3_3_2         0x8032
#define PGM_RENDER_GL_UNSIGNED_BYTE_2_3_3_REV     0x8362
#define PGM_RENDER_GL_UNSIGNED_SHORT_5_6_5        0x8363
#define PGM_RENDER_GL_UNSIGNED_SHORT_5_6_5_REV    0x8364
#define PGM_RENDER_GL_UNSIGNED_SHORT_4_4_4_4      0x8033
#define PGM_RENDER_GL_UNSIGNED_SHORT_4_4_4_4_REV  0x8365
#define PGM_RENDER_GL_UNSIGNED_SHORT_5_5_5_1      0x8034
#define PGM_RENDER_GL_UNSIGNED_SHORT_1_5_5_5_REV  0x8366
#define PGM_RENDER_GL_UNSIGNED_INT_8_8_8_8_REV    0x8367
#define PGM_RENDER_GL_UNSIGNED_INT_10_10_10_2     0x8036
#define PGM_RENDER_GL_UNSIGNED_INT_2_10_10_10_REV 0x8368
#define PGM_RENDER_GL_SMOOTH_LINE_WIDTH_RANGE     0x0B22

#define PGM_RENDER_GL_MODELVIEW  0x1700
#define PGM_RENDER_GL_PROJECTION 0x1701

#define PGM_RENDER_GL_SHORT  0x1402
#define PGM_RENDER_GL_INT    0x1404
#define PGM_RENDER_GL_FLOAT  0x1406
#define PGM_RENDER_GL_DOUBLE 0x140A

#define PGM_RENDER_GL_POINTS         0x0000
#define PGM_RENDER_GL_LINES          0x0001
#define PGM_RENDER_GL_LINE_LOOP      0x0002
#define PGM_RENDER_GL_LINE_STRIP     0x0003
#define PGM_RENDER_GL_TRIANGLES      0x0004
#define PGM_RENDER_GL_TRIANGLE_STRIP 0x0005
#define PGM_RENDER_GL_TRIANGLE_FAN   0x0006
#define PGM_RENDER_GL_QUADS          0x0007
#define PGM_RENDER_GL_QUAD_STRIP     0x0008
#define PGM_RENDER_GL_POLYGON        0x0009

#define PGM_RENDER_GL_VERTEX_ARRAY        0x8074
#define PGM_RENDER_GL_TEXTURE_COORD_ARRAY 0x8078

#define PGM_RENDER_GL_FILL           0x1B02
#define PGM_RENDER_GL_FRONT          0x0404
#define PGM_RENDER_GL_BACK           0x0405
#define PGM_RENDER_GL_CULL_FACE      0x0B44

#define PGM_RENDER_GL_POINT_SMOOTH   0x0B10
#define PGM_RENDER_GL_LINE_SMOOTH    0x0B20
#define PGM_RENDER_GL_POLYGON_SMOOTH 0x0B41

#define PGM_RENDER_GL_SCISSOR_TEST 0x0C11

#define PGM_RENDER_GL_MAX_TEXTURE_SIZE  0x0D33
#define PGM_RENDER_GL_MAX_VIEWPORT_DIMS 0x0D3A

#define PGM_RENDER_GL_TEXTURE_WIDTH        0x1000
#define PGM_RENDER_GL_TEXTURE_HEIGHT       0x1001
#define PGM_RENDER_GL_TEXTURE_BORDER_COLOR 0x1004

#define PGM_RENDER_GL_TEXTURE_BINDING_2D        0x8069
#define PGM_RENDER_GL_TEXTURE_BINDING_RECTANGLE 0x84F6

#define PGM_RENDER_GL_TEXTURE_ENV            0x2300
#define PGM_RENDER_GL_TEXTURE_ENV_MODE       0x2200
#define PGM_RENDER_GL_TEXTURE_2D             0x0DE1
#define PGM_RENDER_GL_PROXY_TEXTURE_2D       0x8064
#define PGM_RENDER_GL_TEXTURE_WRAP_S         0x2802
#define PGM_RENDER_GL_TEXTURE_WRAP_T         0x2803
#define PGM_RENDER_GL_TEXTURE_MAG_FILTER     0x2800
#define PGM_RENDER_GL_TEXTURE_MIN_FILTER     0x2801
#define PGM_RENDER_GL_TEXTURE_ENV_COLOR      0x2201
#define PGM_RENDER_GL_TEXTURE_GEN_S          0x0C60
#define PGM_RENDER_GL_TEXTURE_GEN_T          0x0C61
#define PGM_RENDER_GL_TEXTURE_GEN_MODE       0x2500
#define PGM_RENDER_GL_EYE_LINEAR             0x2400
#define PGM_RENDER_GL_EYE_PLANE              0x2502
#define PGM_RENDER_GL_S                      0x2000
#define PGM_RENDER_GL_T                      0x2001

#define PGM_RENDER_GL_MODULATE               0x2100
#define PGM_RENDER_GL_NEAREST                0x2600
#define PGM_RENDER_GL_LINEAR                 0x2601
#define PGM_RENDER_GL_CLAMP                  0x2900
#define PGM_RENDER_GL_REPEAT                 0x2901
#define PGM_RENDER_GL_CLAMP_TO_EDGE          0x812F
#define PGM_RENDER_GL_CLAMP_TO_BORDER        0x812D
#define PGM_RENDER_GL_TEXTURE_RED_SIZE       0x805C
#define PGM_RENDER_GL_TEXTURE_GREEN_SIZE     0x805D
#define PGM_RENDER_GL_TEXTURE_BLUE_SIZE      0x805E
#define PGM_RENDER_GL_TEXTURE_ALPHA_SIZE     0x805F

#define PGM_RENDER_GL_NEAREST_MIPMAP_NEAREST 0x2700
#define PGM_RENDER_GL_LINEAR_MIPMAP_NEAREST  0x2701
#define PGM_RENDER_GL_NEAREST_MIPMAP_LINEAR  0x2702
#define PGM_RENDER_GL_LINEAR_MIPMAP_LINEAR   0x2703

#define PGM_RENDER_GL_TEXTURE        0x1702
#define PGM_RENDER_GL_SRC_COLOR      0x0300

#define PGM_RENDER_GL_COMBINE        0x8570
#define PGM_RENDER_GL_COMBINE_RGB    0x8571
#define PGM_RENDER_GL_COMBINE_ALPHA  0x8572
#define PGM_RENDER_GL_SOURCE0_RGB    0x8580
#define PGM_RENDER_GL_SOURCE1_RGB    0x8581
#define PGM_RENDER_GL_SOURCE2_RGB    0x8582
#define PGM_RENDER_GL_SOURCE0_ALPHA  0x8588
#define PGM_RENDER_GL_SOURCE1_ALPHA  0x8589
#define PGM_RENDER_GL_SOURCE2_ALPHA  0x858A
#define PGM_RENDER_GL_OPERAND0_RGB   0x8590
#define PGM_RENDER_GL_OPERAND1_RGB   0x8591
#define PGM_RENDER_GL_OPERAND2_RGB   0x8592
#define PGM_RENDER_GL_OPERAND0_ALPHA	0x8598
#define PGM_RENDER_GL_OPERAND1_ALPHA	0x8599
#define PGM_RENDER_GL_OPERAND2_ALPHA	0x859A
#define PGM_RENDER_GL_RGB_SCALE      0x8573
#define PGM_RENDER_GL_ADD_SIGNED     0x8574
#define PGM_RENDER_GL_INTERPOLATE    0x8575
#define PGM_RENDER_GL_SUBTRACT       0x84E7
#define PGM_RENDER_GL_CONSTANT       0x8576
#define PGM_RENDER_GL_PRIMARY_COLOR  0x8577
#define PGM_RENDER_GL_PREVIOUS       0x8578
#define PGM_RENDER_GL_DOT3_RGB       0x86AE
#define PGM_RENDER_GL_DOT3_RGBA      0x86AF

#define PGM_RENDER_GL_STENCIL_TEST 0x0B90
#define PGM_RENDER_GL_KEEP         0x1E00
#define PGM_RENDER_GL_REPLACE      0x1E01
#define PGM_RENDER_GL_INCR         0x1E02
#define PGM_RENDER_GL_DECR         0x1E03

#define PGM_RENDER_GL_LESS       0x0201
#define PGM_RENDER_GL_EQUAL      0x0202
#define PGM_RENDER_GL_LEQUAL     0x0203
#define PGM_RENDER_GL_ALWAYS     0x0207
#define PGM_RENDER_GL_DEPTH_TEST 0x0B71

#define PGM_RENDER_GL_DEPTH_BUFFER_BIT   0x00000100
#define PGM_RENDER_GL_STENCIL_BUFFER_BIT 0x00000400
#define PGM_RENDER_GL_VIEWPORT_BIT       0x00000800
#define PGM_RENDER_GL_TRANSFORM_BIT      0x00001000
#define PGM_RENDER_GL_COLOR_BUFFER_BIT   0x00004000
#define PGM_RENDER_GL_LIST_BIT           0x00020000

#define PGM_RENDER_GL_STENCIL_INDEX   0x1901
#define PGM_RENDER_GL_DEPTH_COMPONENT 0x1902

#define PGM_RENDER_GL_ALPHA           0x1906
#define PGM_RENDER_GL_RGB             0x1907
#define PGM_RENDER_GL_LUMINANCE       0x1909
#define PGM_RENDER_GL_LUMINANCE_ALPHA 0x190A
#define PGM_RENDER_GL_COLOR           0x1800
#define PGM_RENDER_GL_DITHER          0x0BD0
#define PGM_RENDER_GL_RGBA            0x1908
#define PGM_RENDER_GL_BGR             0x80E0
#define PGM_RENDER_GL_BGRA            0x80E1

#define PGM_RENDER_GL_ALPHA4     0x803B
#define PGM_RENDER_GL_ALPHA8     0x803C
#define PGM_RENDER_GL_ALPHA12    0x803D
#define PGM_RENDER_GL_ALPHA16    0x803E
#define PGM_RENDER_GL_LUMINANCE8 0x8040
#define PGM_RENDER_GL_R3_G3_B2   0x2A10
#define PGM_RENDER_GL_RGB4       0x804F
#define PGM_RENDER_GL_RGB5       0x8050
#define PGM_RENDER_GL_RGB8       0x8051
#define PGM_RENDER_GL_RGB10      0x8052
#define PGM_RENDER_GL_RGB12      0x8053
#define PGM_RENDER_GL_RGB16      0x8054
#define PGM_RENDER_GL_RGBA2      0x8055
#define PGM_RENDER_GL_RGBA4      0x8056
#define PGM_RENDER_GL_RGB5_A1    0x8057
#define PGM_RENDER_GL_RGBA8      0x8058
#define PGM_RENDER_GL_RGB10_A2   0x8059
#define PGM_RENDER_GL_RGBA12     0x805A
#define PGM_RENDER_GL_RGBA16     0x805B

#define PGM_RENDER_GL_FRONT_AND_BACK 0x0408
#define PGM_RENDER_GL_FLAT           0x1D00
#define PGM_RENDER_GL_SMOOTH         0x1D01

#define PGM_RENDER_GL_BLEND               0x0BE2
#define PGM_RENDER_GL_ZERO                0x0000
#define PGM_RENDER_GL_ONE                 0x0001
#define PGM_RENDER_GL_ONE_MINUS_SRC_COLOR 0x0301
#define PGM_RENDER_GL_SRC_ALPHA           0x0302
#define PGM_RENDER_GL_ONE_MINUS_SRC_ALPHA 0x0303
#define PGM_RENDER_GL_DST_ALPHA           0x0304
#define PGM_RENDER_GL_ONE_MINUS_DST_ALPHA 0x0305
#define PGM_RENDER_GL_SRC_ALPHA_SATURATE  0x0308
#define PGM_RENDER_GL_CONSTANT_COLOR      0x8001

#define PGM_RENDER_GL_PACK_ALIGNMENT      0x0D05
#define PGM_RENDER_GL_PACK_LSB_FIRST      0x0D01
#define PGM_RENDER_GL_PACK_ROW_LENGTH     0x0D02
#define PGM_RENDER_GL_PACK_SKIP_PIXELS    0x0D04
#define PGM_RENDER_GL_PACK_SKIP_ROWS      0x0D03
#define PGM_RENDER_GL_UNPACK_ALIGNMENT    0x0CF5
#define PGM_RENDER_GL_UNPACK_LSB_FIRST    0x0CF1
#define PGM_RENDER_GL_UNPACK_ROW_LENGTH   0x0CF2
#define PGM_RENDER_GL_UNPACK_SKIP_PIXELS  0x0CF4
#define PGM_RENDER_GL_UNPACK_SKIP_ROWS    0x0CF3

#define PGM_RENDER_GL_PERSPECTIVE_CORRECTION_HINT 0x0C50
#define PGM_RENDER_GL_FASTEST                     0x1101
#define PGM_RENDER_GL_NICEST                      0x1102

#define PGM_RENDER_GL_COMPILE 0x1300

#define PGM_RENDER_GL_TEXTURE_RECTANGLE          0x84F5
#define PGM_RENDER_GL_PROXY_TEXTURE_RECTANGLE    0x84F7
#define PGM_RENDER_GL_MAX_RECTANGLE_TEXTURE_SIZE 0x84F8

#define PGM_RENDER_GL_MIRRORED_REPEAT 0x8370

#define PGM_RENDER_GL_TEXTURE0          0x84C0
#define PGM_RENDER_GL_TEXTURE1          0x84C1
#define PGM_RENDER_GL_TEXTURE2          0x84C2
#define PGM_RENDER_GL_ACTIVE_TEXTURE    0x84E0
#define PGM_RENDER_GL_MAX_TEXTURE_UNITS 0x84E2

#define PGM_RENDER_GL_MULTISAMPLE 0x809D

#define PGM_RENDER_GL_MULTISAMPLE_FILTER_HINT 0x8534

#define PGM_RENDER_GL_FRAGMENT_PROGRAM                    0x8804
#define PGM_RENDER_GL_PROGRAM_STRING                      0x8628
#define PGM_RENDER_GL_PROGRAM_FORMAT_ASCII                0x8875
#define PGM_RENDER_GL_PROGRAM_ERROR_POSITION              0x864B
#define PGM_RENDER_GL_MAX_PROGRAM_LOCAL_PARAMETERS        0x88B4
#define PGM_RENDER_GL_PROGRAM_INSTRUCTIONS                0x88A0
#define PGM_RENDER_GL_MAX_PROGRAM_INSTRUCTIONS            0x88A1
#define PGM_RENDER_GL_PROGRAM_NATIVE_INSTRUCTIONS         0x88A2
#define PGM_RENDER_GL_MAX_PROGRAM_NATIVE_INSTRUCTIONS     0x88A3
#define PGM_RENDER_GL_PROGRAM_PARAMETERS                  0x88A8
#define PGM_RENDER_GL_MAX_PROGRAM_PARAMETERS              0x88A9
#define PGM_RENDER_GL_PROGRAM_NATIVE_PARAMETERS           0x88AA
#define PGM_RENDER_GL_MAX_PROGRAM_NATIVE_PARAMETERS       0x88AB
#define PGM_RENDER_GL_PROGRAM_UNDER_NATIVE_LIMITS         0x88B6
#define PGM_RENDER_GL_PROGRAM_ALU_INSTRUCTIONS            0x8805
#define PGM_RENDER_GL_PROGRAM_TEX_INSTRUCTIONS            0x8806
#define PGM_RENDER_GL_PROGRAM_TEX_INDIRECTIONS            0x8807
#define PGM_RENDER_GL_PROGRAM_NATIVE_ALU_INSTRUCTIONS     0x8808
#define PGM_RENDER_GL_PROGRAM_NATIVE_TEX_INSTRUCTIONS     0x8809
#define PGM_RENDER_GL_PROGRAM_NATIVE_TEX_INDIRECTIONS     0x880A
#define PGM_RENDER_GL_MAX_PROGRAM_ALU_INSTRUCTIONS        0x880B
#define PGM_RENDER_GL_MAX_PROGRAM_TEX_INSTRUCTIONS        0x880C
#define PGM_RENDER_GL_MAX_PROGRAM_TEX_INDIRECTIONS        0x880D
#define PGM_RENDER_GL_MAX_PROGRAM_NATIVE_ALU_INSTRUCTIONS 0x880E
#define PGM_RENDER_GL_MAX_PROGRAM_NATIVE_TEX_INSTRUCTIONS 0x880F
#define PGM_RENDER_GL_MAX_PROGRAM_NATIVE_TEX_INDIRECTIONS 0x8810

#define PGM_RENDER_GL_MAX_TEXTURE_IMAGE_UNITS 0x8872
#define PGM_RENDER_GL_FRAGMENT_SHADER         0x8B30
#define PGM_RENDER_GL_VERTEX_SHADER           0x8B31
#define PGM_RENDER_GL_OBJECT_INFO_LOG_LENGTH  0x8B84
#define PGM_RENDER_GL_PROGRAM_OBJECT          0x8B40
#define PGM_RENDER_GL_OBJECT_DELETE_STATUS    0x8B80
#define PGM_RENDER_GL_OBJECT_COMPILE_STATUS   0x8B81
#define PGM_RENDER_GL_OBJECT_LINK_STATUS      0x8B82
#define PGM_RENDER_GL_OBJECT_VALIDATE_STATUS  0x8B83

#define PGM_RENDER_GL_ARRAY_BUFFER         0x8892
#define PGM_RENDER_GL_PIXEL_PACK_BUFFER    0x88EB
#define PGM_RENDER_GL_PIXEL_UNPACK_BUFFER  0x88EC

#define PGM_RENDER_GL_STREAM_DRAW  0x88E0
#define PGM_RENDER_GL_STREAM_READ  0x88E1
#define PGM_RENDER_GL_STREAM_COPY  0x88E2
#define PGM_RENDER_GL_STATIC_DRAW  0x88E4
#define PGM_RENDER_GL_STATIC_READ  0x88E5
#define PGM_RENDER_GL_STATIC_COPY  0x88E6
#define PGM_RENDER_GL_DYNAMIC_DRAW 0x88E8
#define PGM_RENDER_GL_DYNAMIC_READ 0x88E9
#define PGM_RENDER_GL_DYNAMIC_COPY 0x88EA

#define PGM_RENDER_GL_READ_ONLY  0x88B8
#define PGM_RENDER_GL_WRITE_ONLY 0x88B9
#define PGM_RENDER_GL_READ_WRITE 0x88BA

#define PGM_RENDER_GL_FRAMEBUFFER  0x8D40
#define PGM_RENDER_GL_RENDERBUFFER 0x8D41

#define PGM_RENDER_GL_COLOR_ATTACHMENT0  0x8CE0
#define PGM_RENDER_GL_COLOR_ATTACHMENT1  0x8CE1
#define PGM_RENDER_GL_DEPTH_ATTACHMENT   0x8D00
#define PGM_RENDER_GL_STENCIL_ATTACHMENT 0x8D20

#define PGM_RENDER_GL_FRAMEBUFFER_COMPLETE                        0x8CD5
#define PGM_RENDER_GL_FRAMEBUFFER_INCOMPLETE_ATTACHMENT           0x8CD6
#define PGM_RENDER_GL_FRAMEBUFFER_INCOMPLETE_MISSING_ATTACHMENT   0x8CD7
#define PGM_RENDER_GL_FRAMEBUFFER_INCOMPLETE_DUPLICATE_ATTACHMENT 0x8CD8
#define PGM_RENDER_GL_FRAMEBUFFER_INCOMPLETE_DIMENSIONS           0x8CD9
#define PGM_RENDER_GL_FRAMEBUFFER_INCOMPLETE_FORMATS              0x8CDA
#define PGM_RENDER_GL_FRAMEBUFFER_INCOMPLETE_DRAW_BUFFER          0x8CDB
#define PGM_RENDER_GL_FRAMEBUFFER_INCOMPLETE_READ_BUFFER          0x8CDC
#define PGM_RENDER_GL_FRAMEBUFFER_UNSUPPORTED                     0x8CDD
#define PGM_RENDER_GL_FRAMEBUFFER_STATUS_ERROR                    0x8CDE

#define PGM_RENDER_GL_RENDERBUFFER_RED_SIZE     0x8D50
#define PGM_RENDER_GL_RENDERBUFFER_GREEN_SIZE   0x8D51
#define PGM_RENDER_GL_RENDERBUFFER_BLUE_SIZE    0x8D52
#define PGM_RENDER_GL_RENDERBUFFER_ALPHA_SIZE   0x8D53
#define PGM_RENDER_GL_RENDERBUFFER_DEPTH_SIZE   0x8D54
#define PGM_RENDER_GL_RENDERBUFFER_STENCIL_SIZE 0x8D55

typedef struct _VertexCoord VertexCoord;
typedef struct _Normal Normal;
typedef struct _TextureCoord TextureCoord;
typedef struct _Color Color;

typedef struct _PgmRenderGlProcAddressList PgmRenderGlProcAddressList;
typedef struct _PgmRenderGlExtensionMap PgmRenderGlExtensionMap;
typedef struct _PgmRenderGlBackend PgmRenderGlBackend;
typedef struct _PgmRenderTextureObject PgmRenderTextureObject;
typedef struct _PgmRenderShaderObject PgmRenderShaderObject;

typedef unsigned int PgmRenderGLenum;
typedef unsigned char PgmRenderGLboolean;
typedef char PgmRenderGLchar;
typedef unsigned int PgmRenderGLhandle;
typedef void PgmRenderGLvoid;
typedef int PgmRenderGLint;
typedef unsigned int PgmRenderGLuint;
typedef int PgmRenderGLsizei;
typedef double PgmRenderGLdouble;
typedef float PgmRenderGLfloat;
typedef unsigned short PgmRenderGLushort;
typedef short PgmRenderGLshort;
typedef unsigned int PgmRenderGLbitfield;
typedef double PgmRenderGLclampd;
typedef float PgmRenderGLclampf;
typedef unsigned char PgmRenderGLubyte;
typedef ptrdiff_t PgmRenderGLintptr;
typedef ptrdiff_t PgmRenderGLsizeiptr;

struct _VertexCoord {
  gfloat x;
  gfloat y;
  gfloat z;
  gfloat w;
};

struct _Normal {
  gfloat x;
  gfloat y;
  gfloat z;
};

struct _TextureCoord {
  gfloat s;
  gfloat t;
  gfloat r;
  gfloat q;
};

struct _Color {
  gfloat r;
  gfloat g;
  gfloat b;
  gfloat a;
};

struct _PgmRenderTextureObject {
  gint  size;
  guint *identifiants;
};

struct _PgmRenderGlBackend {
  /* Window management */
  gboolean (*create_window)      (PgmRenderGlBackend *backend);
  void     (*set_title)          (PgmRenderGlBackend *backend, const gchar *title, const gchar *icon);
  gboolean (*destroy_window)     (PgmRenderGlBackend *backend);
  gboolean (*set_pixel_format)   (PgmRenderGlBackend *backend);
  gboolean (*create_context)     (PgmRenderGlBackend *backend);
  gboolean (*destroy_context)    (PgmRenderGlBackend *backend);
  gboolean (*make_current)       (PgmRenderGlBackend *backend);
  void     (*swap_buffers)       (PgmRenderGlBackend *backend);
  gpointer (*get_proc_address)   (PgmRenderGlBackend *backend, const gchar *proc_name);
  gboolean (*check_events)       (PgmRenderGlBackend *backend);
  gboolean (*dispatch_events)    (PgmRenderGlBackend *backend);
  gboolean (*set_size)           (PgmRenderGlBackend *backend, gint width, gint height);
  gboolean (*set_fullscreen)     (PgmRenderGlBackend *backend, gboolean fullscreen, gint width, gint height);
  void     (*get_display_data)   (PgmRenderGlBackend *backend);
  gboolean (*build_text_lists)   (PgmRenderGlBackend *backend);
  void     (*destroy_text_lists) (PgmRenderGlBackend *backend);
  void     (*raster_text)        (PgmRenderGlBackend *backend, const gchar *text, gint x, gint y, gfloat r, gfloat g, gfloat b);
  void     (*wait_for_vblank)    (PgmRenderGlBackend *backend);
  gboolean (*set_cursor)         (PgmRenderGlBackend *backend, PgmRenderCursor cursor);
  gint     event_poll_fd;

  /* Display data */
  gint resolution_width;
  gint resolution_height;
  gint screen_width_mm;
  gint screen_height_mm;

  /* Limits */
  gint   max_texture_2d_size;
  gint   max_texture_rect_size;
  gint   max_texture_units;
  gfloat smooth_line_range[2];

  /* Default pixel stores mode */
  gint row_length;
  gint skip_rows;
  gint skip_pixels;

  /* Informations */
  gfloat                     version;
  const gchar                *vendor;
  const gchar                *version_string;
  const gchar                *renderer;
  const gchar                *extensions;
  PgmRenderGlProcAddressList *proc_address;

  /* Context */
#if defined(G_OS_WIN32)
  PgmRenderWglContext        *context;
#elif defined(G_OS_UNIX)
  PgmRenderGlxContext        *context;
#endif /* defined (G_OS_UNIX) */

  /* Features available at runtime */
  gulong feature_mask;
};

/* OpenGL function types */
typedef PgmRenderGLvoid (PGM_RENDER_GL_API_ATTRIBUTE * pgm_render_gl_enable) (PgmRenderGLenum cap);
typedef PgmRenderGLvoid (PGM_RENDER_GL_API_ATTRIBUTE * pgm_render_gl_disable) (PgmRenderGLenum cap);
typedef PgmRenderGLenum (PGM_RENDER_GL_API_ATTRIBUTE * pgm_render_gl_get_error) (PgmRenderGLvoid);
typedef PgmRenderGLubyte *(PGM_RENDER_GL_API_ATTRIBUTE * pgm_render_gl_get_string) (PgmRenderGLenum name);
typedef PgmRenderGLvoid (PGM_RENDER_GL_API_ATTRIBUTE * pgm_render_gl_begin) (PgmRenderGLenum mode);
typedef PgmRenderGLvoid (PGM_RENDER_GL_API_ATTRIBUTE * pgm_render_gl_end) (PgmRenderGLvoid);
typedef PgmRenderGLvoid (PGM_RENDER_GL_API_ATTRIBUTE * pgm_render_gl_vertex_3f) (PgmRenderGLfloat x, PgmRenderGLfloat y, PgmRenderGLfloat z);
typedef PgmRenderGLvoid (PGM_RENDER_GL_API_ATTRIBUTE * pgm_render_gl_tex_coord_2f) (PgmRenderGLfloat s, PgmRenderGLfloat t);
typedef PgmRenderGLvoid (PGM_RENDER_GL_API_ATTRIBUTE * pgm_render_gl_enable_client_state) (PgmRenderGLenum cap);
typedef PgmRenderGLvoid (PGM_RENDER_GL_API_ATTRIBUTE * pgm_render_gl_disable_client_state) (PgmRenderGLenum cap);
typedef PgmRenderGLvoid (PGM_RENDER_GL_API_ATTRIBUTE * pgm_render_gl_vertex_pointer) (PgmRenderGLint size, PgmRenderGLenum type, PgmRenderGLsizei stride, const PgmRenderGLvoid *ptr);
typedef PgmRenderGLvoid (PGM_RENDER_GL_API_ATTRIBUTE * pgm_render_gl_tex_coord_pointer) (PgmRenderGLint size, PgmRenderGLenum type, PgmRenderGLsizei stride, const PgmRenderGLvoid *ptr);
typedef PgmRenderGLvoid (PGM_RENDER_GL_API_ATTRIBUTE * pgm_render_gl_draw_arrays) (PgmRenderGLenum mode, PgmRenderGLint first, PgmRenderGLsizei count);
typedef PgmRenderGLvoid (PGM_RENDER_GL_API_ATTRIBUTE * pgm_render_gl_multi_draw_arrays) (PgmRenderGLenum mode, PgmRenderGLint *first, PgmRenderGLsizei *count, PgmRenderGLsizei primcount);
typedef PgmRenderGLvoid (PGM_RENDER_GL_API_ATTRIBUTE * pgm_render_gl_tex_env_f) (PgmRenderGLenum target, PgmRenderGLenum pname, PgmRenderGLfloat param);
typedef PgmRenderGLvoid (PGM_RENDER_GL_API_ATTRIBUTE * pgm_render_gl_tex_env_fv) (PgmRenderGLenum target, PgmRenderGLenum pname, const PgmRenderGLfloat *params);
typedef PgmRenderGLvoid (PGM_RENDER_GL_API_ATTRIBUTE * pgm_render_gl_tex_gen_i) (PgmRenderGLenum coord, PgmRenderGLenum pname, PgmRenderGLint param);
typedef PgmRenderGLvoid (PGM_RENDER_GL_API_ATTRIBUTE * pgm_render_gl_tex_gen_fv) (PgmRenderGLenum coord, PgmRenderGLenum pname, const PgmRenderGLfloat *params);
typedef PgmRenderGLvoid (PGM_RENDER_GL_API_ATTRIBUTE * pgm_render_gl_scissor) (PgmRenderGLint x, PgmRenderGLint y, PgmRenderGLsizei width, PgmRenderGLsizei height);
typedef PgmRenderGLvoid (PGM_RENDER_GL_API_ATTRIBUTE * pgm_render_gl_color_4us) (PgmRenderGLushort red, PgmRenderGLushort green, PgmRenderGLushort blue, PgmRenderGLushort alpha);
typedef PgmRenderGLvoid (PGM_RENDER_GL_API_ATTRIBUTE * pgm_render_gl_color_4f) (PgmRenderGLfloat red, PgmRenderGLfloat green, PgmRenderGLfloat blue, PgmRenderGLfloat alpha);
typedef PgmRenderGLvoid (PGM_RENDER_GL_API_ATTRIBUTE * pgm_render_gl_blend_func) (PgmRenderGLenum sfactor, PgmRenderGLenum dfactor);
typedef PgmRenderGLvoid (PGM_RENDER_GL_API_ATTRIBUTE * pgm_render_gl_blend_color) (PgmRenderGLclampf red, PgmRenderGLclampf green, PgmRenderGLclampf blue, PgmRenderGLclampf alpha);
typedef PgmRenderGLvoid (PGM_RENDER_GL_API_ATTRIBUTE * pgm_render_gl_line_width) (PgmRenderGLfloat width);
typedef PgmRenderGLvoid (PGM_RENDER_GL_API_ATTRIBUTE * pgm_render_gl_clear) (PgmRenderGLbitfield mask);
typedef PgmRenderGLvoid (PGM_RENDER_GL_API_ATTRIBUTE * pgm_render_gl_clear_color) (PgmRenderGLclampf red, PgmRenderGLclampf green, PgmRenderGLclampf blue, PgmRenderGLclampf alpha);
typedef PgmRenderGLvoid (PGM_RENDER_GL_API_ATTRIBUTE * pgm_render_gl_clear_stencil) (PgmRenderGLint s);
typedef PgmRenderGLvoid (PGM_RENDER_GL_API_ATTRIBUTE * pgm_render_gl_stencil_func) (PgmRenderGLenum func, PgmRenderGLint ref, PgmRenderGLuint mask);
typedef PgmRenderGLvoid (PGM_RENDER_GL_API_ATTRIBUTE * pgm_render_gl_stencil_op) (PgmRenderGLenum fail, PgmRenderGLenum zfail, PgmRenderGLenum zpass);
typedef PgmRenderGLvoid (PGM_RENDER_GL_API_ATTRIBUTE * pgm_render_gl_push_attrib) (PgmRenderGLbitfield mask);
typedef PgmRenderGLvoid (PGM_RENDER_GL_API_ATTRIBUTE * pgm_render_gl_pop_attrib) (PgmRenderGLvoid);
typedef PgmRenderGLvoid (PGM_RENDER_GL_API_ATTRIBUTE * pgm_render_gl_matrix_mode) (PgmRenderGLenum mode);
typedef PgmRenderGLvoid (PGM_RENDER_GL_API_ATTRIBUTE * pgm_render_gl_push_matrix) (PgmRenderGLvoid);
typedef PgmRenderGLvoid (PGM_RENDER_GL_API_ATTRIBUTE * pgm_render_gl_pop_matrix) (PgmRenderGLvoid);
typedef PgmRenderGLvoid (PGM_RENDER_GL_API_ATTRIBUTE * pgm_render_gl_load_identity) (PgmRenderGLvoid);
typedef PgmRenderGLvoid (PGM_RENDER_GL_API_ATTRIBUTE * pgm_render_gl_load_matrix_f) (const PgmRenderGLfloat *m);
typedef PgmRenderGLvoid (PGM_RENDER_GL_API_ATTRIBUTE * pgm_render_gl_depth_range) (PgmRenderGLclampd near_val, PgmRenderGLclampd far_val);
typedef PgmRenderGLvoid (PGM_RENDER_GL_API_ATTRIBUTE * pgm_render_gl_viewport) (PgmRenderGLint x, PgmRenderGLint y, PgmRenderGLsizei width, PgmRenderGLsizei height);
typedef PgmRenderGLvoid (PGM_RENDER_GL_API_ATTRIBUTE * pgm_render_gl_raster_pos_2f) (PgmRenderGLfloat x, PgmRenderGLfloat y);
typedef PgmRenderGLvoid (PGM_RENDER_GL_API_ATTRIBUTE * pgm_render_gl_bitmap) (PgmRenderGLsizei width, PgmRenderGLsizei height, PgmRenderGLfloat xorig, PgmRenderGLfloat yorig, PgmRenderGLfloat xmove, PgmRenderGLfloat ymove, const PgmRenderGLubyte *bitmap);
typedef PgmRenderGLvoid (PGM_RENDER_GL_API_ATTRIBUTE * pgm_render_gl_read_buffer) (PgmRenderGLenum mode);
typedef PgmRenderGLvoid (PGM_RENDER_GL_API_ATTRIBUTE * pgm_render_gl_draw_buffer) (PgmRenderGLenum mode);
typedef PgmRenderGLvoid (PGM_RENDER_GL_API_ATTRIBUTE * pgm_render_gl_copy_pixels) (PgmRenderGLint x, PgmRenderGLint y, PgmRenderGLsizei width, PgmRenderGLsizei height, PgmRenderGLenum type);
typedef PgmRenderGLvoid (PGM_RENDER_GL_API_ATTRIBUTE * pgm_render_gl_flush) (PgmRenderGLvoid);
typedef PgmRenderGLvoid (PGM_RENDER_GL_API_ATTRIBUTE * pgm_render_gl_finish) (PgmRenderGLvoid);
typedef PgmRenderGLvoid (PGM_RENDER_GL_API_ATTRIBUTE * pgm_render_gl_pixel_store_i) (PgmRenderGLenum pname, PgmRenderGLint param);
typedef PgmRenderGLvoid (PGM_RENDER_GL_API_ATTRIBUTE * pgm_render_gl_frustum) (PgmRenderGLdouble left, PgmRenderGLdouble right, PgmRenderGLdouble bottom, PgmRenderGLdouble top, PgmRenderGLdouble near_val, PgmRenderGLdouble far_val);
typedef PgmRenderGLvoid (PGM_RENDER_GL_API_ATTRIBUTE * pgm_render_gl_ortho) (PgmRenderGLdouble left, PgmRenderGLdouble right, PgmRenderGLdouble bottom, PgmRenderGLdouble top, PgmRenderGLdouble near_val, PgmRenderGLdouble far_val);
typedef PgmRenderGLvoid (PGM_RENDER_GL_API_ATTRIBUTE * pgm_render_gl_scale_f) (PgmRenderGLfloat x, PgmRenderGLfloat y, PgmRenderGLfloat z);
typedef PgmRenderGLvoid (PGM_RENDER_GL_API_ATTRIBUTE * pgm_render_gl_translate_f) (PgmRenderGLfloat x, PgmRenderGLfloat y, PgmRenderGLfloat z);
typedef PgmRenderGLvoid (PGM_RENDER_GL_API_ATTRIBUTE * pgm_render_gl_rotate_f) (PgmRenderGLfloat angle, PgmRenderGLfloat x, PgmRenderGLfloat y, PgmRenderGLfloat z);

/* Display lists */
typedef PgmRenderGLboolean (PGM_RENDER_GL_API_ATTRIBUTE * pgm_render_gl_is_list) (PgmRenderGLuint list);
typedef PgmRenderGLvoid (PGM_RENDER_GL_API_ATTRIBUTE * pgm_render_gl_delete_lists) (PgmRenderGLuint list, PgmRenderGLsizei range);
typedef PgmRenderGLuint (PGM_RENDER_GL_API_ATTRIBUTE * pgm_render_gl_gen_lists) (PgmRenderGLsizei range);
typedef PgmRenderGLvoid (PGM_RENDER_GL_API_ATTRIBUTE * pgm_render_gl_new_list) (PgmRenderGLuint list, PgmRenderGLenum mode);
typedef PgmRenderGLvoid (PGM_RENDER_GL_API_ATTRIBUTE * pgm_render_gl_end_list) (PgmRenderGLvoid);
typedef PgmRenderGLvoid (PGM_RENDER_GL_API_ATTRIBUTE * pgm_render_gl_call_list) (PgmRenderGLuint list);
typedef PgmRenderGLvoid (PGM_RENDER_GL_API_ATTRIBUTE * pgm_render_gl_call_lists) (PgmRenderGLsizei n, PgmRenderGLenum type, const PgmRenderGLvoid *lists);
typedef PgmRenderGLvoid (PGM_RENDER_GL_API_ATTRIBUTE * pgm_render_gl_list_base) (PgmRenderGLuint base);

typedef PgmRenderGLvoid (PGM_RENDER_GL_API_ATTRIBUTE * pgm_render_gl_hint) (PgmRenderGLenum target, PgmRenderGLenum mode);
typedef PgmRenderGLvoid (PGM_RENDER_GL_API_ATTRIBUTE * pgm_render_gl_depth_mask) (PgmRenderGLboolean flag);
typedef PgmRenderGLvoid (PGM_RENDER_GL_API_ATTRIBUTE * pgm_render_gl_polygon_mode) (PgmRenderGLenum face, PgmRenderGLenum mode);
typedef PgmRenderGLvoid (PGM_RENDER_GL_API_ATTRIBUTE * pgm_render_gl_shade_model) (PgmRenderGLenum mode);
typedef PgmRenderGLvoid (PGM_RENDER_GL_API_ATTRIBUTE * pgm_render_gl_color_mask) (PgmRenderGLboolean red, PgmRenderGLboolean green, PgmRenderGLboolean blue, PgmRenderGLboolean alpha);
typedef PgmRenderGLvoid (PGM_RENDER_GL_API_ATTRIBUTE * pgm_render_gl_read_pixels) (PgmRenderGLint x, PgmRenderGLint y, PgmRenderGLsizei width, PgmRenderGLsizei height, PgmRenderGLenum format, PgmRenderGLenum type, PgmRenderGLvoid *pixels);
typedef PgmRenderGLvoid (PGM_RENDER_GL_API_ATTRIBUTE * pgm_render_gl_get_tex_image) (PgmRenderGLenum target, PgmRenderGLint level, PgmRenderGLenum format, PgmRenderGLenum type, PgmRenderGLvoid *pixels);
typedef PgmRenderGLvoid (PGM_RENDER_GL_API_ATTRIBUTE * pgm_render_gl_tex_sub_image_2d)    (PgmRenderGLenum target, PgmRenderGLint level, PgmRenderGLint xoffset, PgmRenderGLint yoffset, PgmRenderGLsizei width, PgmRenderGLsizei height, PgmRenderGLenum format, PgmRenderGLenum type, const PgmRenderGLvoid *pixels);
typedef PgmRenderGLvoid (PGM_RENDER_GL_API_ATTRIBUTE * pgm_render_gl_gen_textures) (PgmRenderGLsizei n, PgmRenderGLuint *textures);
typedef PgmRenderGLvoid (PGM_RENDER_GL_API_ATTRIBUTE * pgm_render_gl_delete_textures) (PgmRenderGLsizei n, const PgmRenderGLuint *textures);
typedef PgmRenderGLvoid (PGM_RENDER_GL_API_ATTRIBUTE * pgm_render_gl_bind_texture) (PgmRenderGLenum target, PgmRenderGLuint texture);
typedef PgmRenderGLvoid (PGM_RENDER_GL_API_ATTRIBUTE * pgm_render_gl_tex_image_2d) (PgmRenderGLenum target, PgmRenderGLint level, PgmRenderGLint internal_format, PgmRenderGLsizei width, PgmRenderGLsizei height, PgmRenderGLint border, PgmRenderGLenum format, PgmRenderGLenum type, const PgmRenderGLvoid *pixels);
typedef PgmRenderGLvoid (PGM_RENDER_GL_API_ATTRIBUTE * pgm_render_gl_tex_parameter_i) (PgmRenderGLenum target, PgmRenderGLenum pname, PgmRenderGLint param);
typedef PgmRenderGLvoid (PGM_RENDER_GL_API_ATTRIBUTE * pgm_render_gl_tex_parameter_fv) (PgmRenderGLenum target, PgmRenderGLenum pname, const PgmRenderGLfloat *param);
typedef PgmRenderGLvoid (PGM_RENDER_GL_API_ATTRIBUTE * pgm_render_gl_get_tex_level_parameter_iv) (PgmRenderGLenum target, PgmRenderGLint level, PgmRenderGLenum pname, PgmRenderGLint *param);
typedef PgmRenderGLvoid (PGM_RENDER_GL_API_ATTRIBUTE * pgm_render_gl_copy_tex_sub_image_2d) (PgmRenderGLenum target, PgmRenderGLint level, PgmRenderGLint xoffset, PgmRenderGLint yoffset, PgmRenderGLint x, PgmRenderGLint y, PgmRenderGLsizei width, PgmRenderGLsizei height);
typedef PgmRenderGLvoid (PGM_RENDER_GL_API_ATTRIBUTE * pgm_render_gl_get_integer_v) (PgmRenderGLenum pname, PgmRenderGLint *params);
typedef PgmRenderGLvoid (PGM_RENDER_GL_API_ATTRIBUTE * pgm_render_gl_get_float_v) (PgmRenderGLenum pname, PgmRenderGLfloat *params);
typedef PgmRenderGLvoid (PGM_RENDER_GL_API_ATTRIBUTE * pgm_render_gl_get_pointer_v) (PgmRenderGLenum pname, PgmRenderGLvoid **params);
typedef PgmRenderGLvoid (PGM_RENDER_GL_API_ATTRIBUTE * pgm_render_gl_active_texture) (PgmRenderGLenum);
typedef PgmRenderGLvoid (PGM_RENDER_GL_API_ATTRIBUTE * pgm_render_gl_client_active_texture) (PgmRenderGLenum);
typedef PgmRenderGLvoid (PGM_RENDER_GL_API_ATTRIBUTE * pgm_render_gl_gen_programs) (PgmRenderGLsizei, PgmRenderGLuint *);
typedef PgmRenderGLvoid (PGM_RENDER_GL_API_ATTRIBUTE * pgm_render_gl_delete_programs) (PgmRenderGLsizei, const PgmRenderGLuint *);
typedef PgmRenderGLvoid (PGM_RENDER_GL_API_ATTRIBUTE * pgm_render_gl_program_string) (PgmRenderGLenum, PgmRenderGLenum, PgmRenderGLsizei, const PgmRenderGLvoid *);
typedef PgmRenderGLvoid (PGM_RENDER_GL_API_ATTRIBUTE * pgm_render_gl_bind_program) (PgmRenderGLenum, PgmRenderGLuint);
typedef PgmRenderGLvoid (PGM_RENDER_GL_API_ATTRIBUTE * pgm_render_gl_program_local_param_4fv) (PgmRenderGLenum, PgmRenderGLuint, const PgmRenderGLfloat *);
typedef PgmRenderGLvoid (PGM_RENDER_GL_API_ATTRIBUTE * pgm_render_gl_get_program_iv) (PgmRenderGLenum, PgmRenderGLenum, PgmRenderGLint *);
typedef PgmRenderGLvoid (PGM_RENDER_GL_API_ATTRIBUTE * pgm_render_gl_gen_buffers) (PgmRenderGLsizei, PgmRenderGLuint *buffers);
typedef PgmRenderGLvoid (PGM_RENDER_GL_API_ATTRIBUTE * pgm_render_gl_delete_buffers) (PgmRenderGLsizei, const PgmRenderGLuint *buffers);
typedef PgmRenderGLvoid (PGM_RENDER_GL_API_ATTRIBUTE * pgm_render_gl_bind_buffer) (PgmRenderGLenum, PgmRenderGLuint buffer);
typedef PgmRenderGLvoid (PGM_RENDER_GL_API_ATTRIBUTE * pgm_render_gl_buffer_data) (PgmRenderGLenum, PgmRenderGLsizeiptr, const PgmRenderGLvoid *, PgmRenderGLenum);
typedef PgmRenderGLvoid *(PGM_RENDER_GL_API_ATTRIBUTE * pgm_render_gl_buffer_sub_data) (PgmRenderGLenum, PgmRenderGLintptr, PgmRenderGLsizeiptr, const PgmRenderGLvoid *);
typedef PgmRenderGLvoid *(PGM_RENDER_GL_API_ATTRIBUTE * pgm_render_gl_get_buffer_sub_data) (PgmRenderGLenum, PgmRenderGLintptr, PgmRenderGLsizeiptr, PgmRenderGLvoid *);
typedef PgmRenderGLvoid *(PGM_RENDER_GL_API_ATTRIBUTE * pgm_render_gl_map_buffer) (PgmRenderGLenum, PgmRenderGLenum);
typedef PgmRenderGLboolean (PGM_RENDER_GL_API_ATTRIBUTE * pgm_render_gl_unmap_buffer) (PgmRenderGLenum);
typedef PgmRenderGLvoid (PGM_RENDER_GL_API_ATTRIBUTE * pgm_render_gl_gen_framebuffers) (PgmRenderGLsizei, PgmRenderGLuint *);
typedef PgmRenderGLvoid (PGM_RENDER_GL_API_ATTRIBUTE * pgm_render_gl_delete_framebuffers) (PgmRenderGLsizei, const PgmRenderGLuint *);
typedef PgmRenderGLvoid (PGM_RENDER_GL_API_ATTRIBUTE * pgm_render_gl_bind_framebuffer) (PgmRenderGLenum, PgmRenderGLuint);
typedef PgmRenderGLvoid (PGM_RENDER_GL_API_ATTRIBUTE * pgm_render_gl_framebuffer_renderbuffer) (PgmRenderGLenum, PgmRenderGLenum, PgmRenderGLenum, PgmRenderGLuint);
typedef PgmRenderGLvoid (PGM_RENDER_GL_API_ATTRIBUTE * pgm_render_gl_framebuffer_texture_2d) (PgmRenderGLenum, PgmRenderGLenum, PgmRenderGLenum, PgmRenderGLuint, PgmRenderGLint);
typedef PgmRenderGLenum (PGM_RENDER_GL_API_ATTRIBUTE * pgm_render_gl_check_framebuffer_status) (PgmRenderGLenum);
typedef PgmRenderGLvoid (PGM_RENDER_GL_API_ATTRIBUTE * pgm_render_gl_gen_renderbuffers) (PgmRenderGLsizei, PgmRenderGLuint *);
typedef PgmRenderGLvoid (PGM_RENDER_GL_API_ATTRIBUTE * pgm_render_gl_delete_renderbuffers) (PgmRenderGLsizei, const PgmRenderGLuint *);
typedef PgmRenderGLvoid (PGM_RENDER_GL_API_ATTRIBUTE * pgm_render_gl_bind_renderbuffer) (PgmRenderGLenum, PgmRenderGLuint);
typedef PgmRenderGLvoid (PGM_RENDER_GL_API_ATTRIBUTE * pgm_render_gl_renderbuffer_storage) (PgmRenderGLenum, PgmRenderGLenum, PgmRenderGLsizei, PgmRenderGLsizei);
typedef PgmRenderGLvoid (PGM_RENDER_GL_API_ATTRIBUTE * pgm_render_gl_get_renderbuffer_parameter_iv) (PgmRenderGLenum, PgmRenderGLenum, PgmRenderGLint *);
typedef PgmRenderGLvoid (PGM_RENDER_GL_API_ATTRIBUTE * pgm_render_gl_delete_object) (PgmRenderGLhandle);
typedef PgmRenderGLhandle (PGM_RENDER_GL_API_ATTRIBUTE * pgm_render_gl_get_handle) (PgmRenderGLenum);
typedef PgmRenderGLvoid (PGM_RENDER_GL_API_ATTRIBUTE * pgm_render_gl_detach_object) (PgmRenderGLhandle, PgmRenderGLhandle);
typedef PgmRenderGLhandle (PGM_RENDER_GL_API_ATTRIBUTE * pgm_render_gl_create_shader_object) (PgmRenderGLenum);
typedef PgmRenderGLvoid (PGM_RENDER_GL_API_ATTRIBUTE * pgm_render_gl_shader_source) (PgmRenderGLhandle, PgmRenderGLsizei, const PgmRenderGLchar **, const PgmRenderGLint *);
typedef PgmRenderGLvoid (PGM_RENDER_GL_API_ATTRIBUTE * pgm_render_gl_compile_shader) (PgmRenderGLhandle);
typedef PgmRenderGLhandle (PGM_RENDER_GL_API_ATTRIBUTE * pgm_render_gl_create_program_object) (PgmRenderGLvoid);
typedef PgmRenderGLvoid (PGM_RENDER_GL_API_ATTRIBUTE * pgm_render_gl_attach_object) (PgmRenderGLhandle, PgmRenderGLhandle);
typedef PgmRenderGLvoid (PGM_RENDER_GL_API_ATTRIBUTE * pgm_render_gl_link_program) (PgmRenderGLhandle);
typedef PgmRenderGLvoid (PGM_RENDER_GL_API_ATTRIBUTE * pgm_render_gl_use_program_object) (PgmRenderGLhandle);
typedef PgmRenderGLvoid (PGM_RENDER_GL_API_ATTRIBUTE * pgm_render_gl_get_object_parameter_iv) (PgmRenderGLhandle, PgmRenderGLenum, PgmRenderGLint *);
typedef PgmRenderGLvoid (PGM_RENDER_GL_API_ATTRIBUTE * pgm_render_gl_get_info_log) (PgmRenderGLhandle, PgmRenderGLsizei, PgmRenderGLsizei *, PgmRenderGLchar *);
typedef PgmRenderGLint (PGM_RENDER_GL_API_ATTRIBUTE * pgm_render_gl_get_uniform_location) (PgmRenderGLhandle, const PgmRenderGLchar *);
typedef PgmRenderGLvoid (PGM_RENDER_GL_API_ATTRIBUTE * pgm_render_gl_uniform_matrix_4fv) (PgmRenderGLint, PgmRenderGLsizei, PgmRenderGLboolean, const PgmRenderGLfloat *);
typedef PgmRenderGLvoid (PGM_RENDER_GL_API_ATTRIBUTE * pgm_render_gl_uniform_1f) (PgmRenderGLint, PgmRenderGLfloat);
typedef PgmRenderGLvoid (PGM_RENDER_GL_API_ATTRIBUTE * pgm_render_gl_uniform_1i) (PgmRenderGLint, PgmRenderGLint);

struct _PgmRenderGlProcAddressList {
  /* OpenGL core */
  pgm_render_gl_enable                     enable;
  pgm_render_gl_disable                    disable;
  pgm_render_gl_get_error                  get_error;
  pgm_render_gl_get_string                 get_string;
  pgm_render_gl_begin                      begin;
  pgm_render_gl_end                        end;
  pgm_render_gl_vertex_3f                  vertex_3f;
  pgm_render_gl_tex_coord_2f               tex_coord_2f;
  pgm_render_gl_enable_client_state        enable_client_state;
  pgm_render_gl_disable_client_state       disable_client_state;
  pgm_render_gl_vertex_pointer             vertex_pointer;
  pgm_render_gl_tex_coord_pointer          tex_coord_pointer;
  pgm_render_gl_draw_arrays                draw_arrays;
  pgm_render_gl_tex_env_f                  tex_env_f;
  pgm_render_gl_tex_env_fv                 tex_env_fv;
  pgm_render_gl_tex_gen_i                  tex_gen_i;
  pgm_render_gl_tex_gen_fv                 tex_gen_fv;
  pgm_render_gl_color_4us                  color_4us;
  pgm_render_gl_color_4f                   color_4f;
  pgm_render_gl_scissor                    scissor;
  pgm_render_gl_blend_func                 blend_func;
  pgm_render_gl_line_width                 line_width;
  pgm_render_gl_clear                      clear;
  pgm_render_gl_clear_color                clear_color;
  pgm_render_gl_clear_stencil              clear_stencil;
  pgm_render_gl_stencil_func               stencil_func;
  pgm_render_gl_stencil_op                 stencil_op;
  pgm_render_gl_push_attrib                push_attrib;
  pgm_render_gl_pop_attrib                 pop_attrib;
  pgm_render_gl_matrix_mode                matrix_mode;
  pgm_render_gl_push_matrix                push_matrix;
  pgm_render_gl_pop_matrix                 pop_matrix;
  pgm_render_gl_load_identity              load_identity;
  pgm_render_gl_load_matrix_f              load_matrix_f;
  pgm_render_gl_depth_range                depth_range;
  pgm_render_gl_viewport                   viewport;
  pgm_render_gl_raster_pos_2f              raster_pos_2f;
  pgm_render_gl_bitmap                     bitmap;
  pgm_render_gl_read_buffer                read_buffer;
  pgm_render_gl_draw_buffer                draw_buffer;
  pgm_render_gl_copy_pixels                copy_pixels;
  pgm_render_gl_flush                      flush;
  pgm_render_gl_finish                     finish;
  pgm_render_gl_pixel_store_i              pixel_store_i;
  pgm_render_gl_frustum                    frustum;
  pgm_render_gl_ortho                      ortho;
  pgm_render_gl_scale_f                    scale_f;
  pgm_render_gl_translate_f                translate_f;
  pgm_render_gl_rotate_f                   rotate_f;
  pgm_render_gl_is_list                    is_list;
  pgm_render_gl_delete_lists               delete_lists;
  pgm_render_gl_gen_lists                  gen_lists;
  pgm_render_gl_new_list                   new_list;
  pgm_render_gl_end_list                   end_list;
  pgm_render_gl_call_list                  call_list;
  pgm_render_gl_call_lists                 call_lists;
  pgm_render_gl_list_base                  list_base;
  pgm_render_gl_hint                       hint;
  pgm_render_gl_depth_mask                 depth_mask;
  pgm_render_gl_polygon_mode               polygon_mode;
  pgm_render_gl_shade_model                shade_model;
  pgm_render_gl_color_mask                 color_mask;
  pgm_render_gl_read_pixels                read_pixels;
  pgm_render_gl_get_tex_image              get_tex_image;
  pgm_render_gl_tex_sub_image_2d           tex_sub_image_2d;
  pgm_render_gl_gen_textures               gen_textures;
  pgm_render_gl_delete_textures            delete_textures;
  pgm_render_gl_bind_texture               bind_texture;
  pgm_render_gl_tex_image_2d               tex_image_2d;
  pgm_render_gl_tex_parameter_i            tex_parameter_i;
  pgm_render_gl_tex_parameter_fv           tex_parameter_fv;
  pgm_render_gl_get_tex_level_parameter_iv get_tex_level_parameter_iv;
  pgm_render_gl_copy_tex_sub_image_2d      copy_tex_sub_image_2d;
  pgm_render_gl_get_integer_v              get_integer_v;
  pgm_render_gl_get_float_v                get_float_v;

  /* OpenGL extensions */
  pgm_render_gl_blend_color                   blend_color;
  pgm_render_gl_active_texture                active_texture;
  pgm_render_gl_client_active_texture         client_active_texture;
  pgm_render_gl_multi_draw_arrays             multi_draw_arrays;
  pgm_render_gl_gen_programs                  gen_programs;
  pgm_render_gl_delete_programs               delete_programs;
  pgm_render_gl_program_string                program_string;
  pgm_render_gl_bind_program                  bind_program;
  pgm_render_gl_program_local_param_4fv       program_local_param_4fv;
  pgm_render_gl_get_program_iv                get_program_iv;
  pgm_render_gl_gen_buffers                   gen_buffers;
  pgm_render_gl_delete_buffers                delete_buffers;
  pgm_render_gl_bind_buffer                   bind_buffer;
  pgm_render_gl_buffer_data                   buffer_data;
  pgm_render_gl_buffer_sub_data               buffer_sub_data;
  pgm_render_gl_get_buffer_sub_data           get_buffer_sub_data;
  pgm_render_gl_map_buffer                    map_buffer;
  pgm_render_gl_unmap_buffer                  unmap_buffer;
  pgm_render_gl_gen_framebuffers              gen_framebuffers;
  pgm_render_gl_delete_framebuffers           delete_framebuffers;
  pgm_render_gl_bind_framebuffer              bind_framebuffer;
  pgm_render_gl_framebuffer_renderbuffer      framebuffer_renderbuffer;
  pgm_render_gl_framebuffer_texture_2d        framebuffer_texture_2d;
  pgm_render_gl_check_framebuffer_status      check_framebuffer_status;
  pgm_render_gl_gen_renderbuffers             gen_renderbuffers;
  pgm_render_gl_delete_renderbuffers          delete_renderbuffers;
  pgm_render_gl_bind_renderbuffer             bind_renderbuffer;
  pgm_render_gl_renderbuffer_storage          renderbuffer_storage;
  pgm_render_gl_get_renderbuffer_parameter_iv get_renderbuffer_parameter_iv;
  pgm_render_gl_delete_object                 delete_object;
  pgm_render_gl_get_handle                    get_handle;
  pgm_render_gl_detach_object                 detach_object;
  pgm_render_gl_create_shader_object          create_shader_object;
  pgm_render_gl_shader_source                 shader_source;
  pgm_render_gl_compile_shader                compile_shader;
  pgm_render_gl_create_program_object         create_program_object;
  pgm_render_gl_attach_object                 attach_object;
  pgm_render_gl_link_program                  link_program;
  pgm_render_gl_use_program_object            use_program_object;
  pgm_render_gl_get_object_parameter_iv       get_object_parameter_iv;
  pgm_render_gl_get_info_log                  get_info_log;
  pgm_render_gl_get_uniform_location          get_uniform_location;
  pgm_render_gl_uniform_matrix_4fv            uniform_matrix_4fv;
  pgm_render_gl_uniform_1f                    uniform_1f;
  pgm_render_gl_uniform_1i                    uniform_1i;
};

struct _PgmRenderGlExtensionMap {
  gchar *name;
  gint   mask;
};

#endif /* __PGM_RENDER_GL_H__ */
