/* -*- Mode: C; c-basic-offset: 2; indent-tabs-mode: nil -*-
 *
 * Pigment Render OpenGL 1.2 plugin
 *
 * Copyright © 2006, 2007 Fluendo Embedded S.L. <www.fluendo.com>
 * All rights reserved.
 *
 * This file is available under one of two license agreements.
 *
 * This file is licensed under the GPL version 2. See "LICENSE.GPL" in
 * the root of this distribution including a special exception to use
 * Pigment with Fluendo's plugins.
 *
 * The GPL part of Pigment is also available under a commercial licensing
 * agreement from Fluendo. See "LICENSE.Pigment" in the root directory
 * of this distribution package for details on that license.
 *
 * Author: Loïc Molinari <loic@fluendo.com>
 */

#ifndef __PGM_RENDER_GL1_WINDOW_H__
#define __PGM_RENDER_GL1_WINDOW_H__

/* FIXME: Correct use of the autotools */
#include "pgmrendergl1.h"
#include "pgmrendergl1surface.h"
#include "pgmrendergl1font.h"

G_BEGIN_DECLS

#define PGM_RENDER_TYPE_GL1_WINDOW            (pgm_render_gl1_window_get_type())
#define PGM_RENDER_GL1_WINDOW(obj)            (G_TYPE_CHECK_INSTANCE_CAST((obj), PGM_RENDER_TYPE_GL1_WINDOW, PgmRenderGl1Window))
#define PGM_RENDER_GL1_WINDOW_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST((klass), PGM_RENDER_TYPE_GL1_WINDOW, BaseWindow))
#define PGM_RENDER_IS_GL1_WINDOW(obj)         (G_TYPE_CHECK_INSTANCE_TYPE((obj), PGM_RENDER_TYPE_GL1_WINDOW))
#define PGM_RENDER_IS_GL1_WINDOW_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE((klass), PGM_RENDER_TYPE_GL1_WINDOW))
#define PGM_RENDER_GL1_WINDOW_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS((obj), PGM_RENDER_TYPE_GL1_WINDOW, PgmRenderGl1WindowClass))

#define DUMP_GPU_SUPPORT_ENV_NAME "PGM_DUMP_GPU_SUPPORT"
#define SHOW_FPS_ENV_NAME         "PGM_SHOW_FPS"
/* FIXME: Temporary environment variable to correct broken X screen
 *        size retrieving. */
#define BROKEN_X_SIZE_ENV_NAME    "PGM_BROKEN_X_SIZE"
#define NB_COLOR_SPACES           8
#define NB_INTERP_TYPES           2

/* typedef struct _PgmRenderGl1Window PgmRenderGl1Window; */
typedef struct _PgmRenderGl1WindowClass PgmRenderGl1WindowClass;
typedef struct _FragmentCode FragmentCode;
typedef struct _ColorSpaceShader ColorSpaceShader;

typedef void (*CreateTextureFunc) (gint width, gint height, guint *tex_id,
                                   PgmRenderGLenum filter,
                                   PgmRenderGlProcAddressList *gl);
typedef void (*UploadTextureFunc) (gint width, gint height, gint stride,
                                   gint row_lenght, gint skip_rows,
                                   gint skip_pixels, guint *tex_id, void *data,
                                   PgmRenderGlProcAddressList *gl);

struct _PgmRenderEventSource {
  GSource source;
  GPollFD event_poll_fd;
};

struct _FragmentCode {
  const gchar **sampler;
  gchar *nearest_func_name;
  gchar *bilinear_func_name;
};

struct _ColorSpaceShader {
  PgmRenderGLhandle program;
  PgmRenderGLint    tex_loc[3];
  PgmRenderGLint    tex_width_loc;
  PgmRenderGLint    tex_height_loc;
};

struct _PgmRenderGl1Window {
  PgmRenderBaseWindow parent;

  GMutex              *context_creation_mutex;
  GCond               *context_creation_cond;
  GMutex              *start_loop_mutex;
  GCond               *start_loop_cond;
  GThread             *loop_thread;
  gboolean            loop_started;
  gboolean            context_created;
  GList               *event;
  gchar               *window_title;
  gchar               *icon_title;
  GMainContext        *context;
  GMainLoop           *loop;
  gboolean            fullscreen;
  gboolean            new_size;
  gboolean            new_canvas_size;
  gboolean            new_refresh_color;
  gboolean            new_cursor;
  Color               color;
  GSource             *source;
  guint               source_id;
  PgmRenderGlBackend  *backend;
  ColorSpaceShader    color_space_shader[NB_COLOR_SPACES][NB_INTERP_TYPES];
  CreateTextureFunc   create_texture[NB_COLOR_SPACES];
  UploadTextureFunc   upload_texture[NB_COLOR_SPACES];
  PgmRenderGLenum     creation_filter;
  gboolean            new_scale;
  gboolean            new_title;
  gboolean            fullscreen_request;
  GTimeVal            update_time;
  GTimeVal            fps_print_time;
  gboolean            show_fps;
  GList               *surface;
  GList               *font;
};

struct _PgmRenderGl1WindowClass {
  PgmRenderBaseWindowClass parent_class;

  PgmRenderGl1SurfaceClass *surface_class;
  PgmRenderGl1FontClass    *font_class;
};

GType               pgm_render_gl1_window_get_type (void) G_GNUC_CONST;
PgmRenderGl1Window *pgm_render_gl1_window_new      (void);

G_END_DECLS

#endif /* __PGM_RENDER_GL1_WINDOW_H__ */
